unit ILMessages;
{Version 1.0}

interface

var
  LastErrText : string;
  Comment1, Comment2, Comment3 : string;

{ -------------------------------------------------------------------------
  This unit contains error messages and terms displayed by ILIB.
  Each message is a resource string constant starting with a prefix:

  UT_ : User terms, that means words or phrases meant for the user
  UE_ : User errors: erros the user may make when operating the aplication
  DE_ : Developer errors: illegalties committed by the developer.

  Additionally each message is identified by a constant number, prefixed
  as follows:
  T_ : Terms
  E_ : Errors (both user and developer errors

  The messages are organized in string arrays whos entries contain
  the string constants. When handling messages, ILIB looks them up in the array
  by their constant numbers.
  The arrays are:

  T_TERMS
  E_ERRORTEXT (user errors)
  E_INTERNALS (developer errors)

  The developer can use the resource strings to translate messages and
  terms by menas of the Translation manager in the Delphi IDE.

  Also it is possible to translate field names and table names. ILMessages
  declares two dynamic arrays: TranslateFieldCaps and TranslateTableCaps that
  the developer can populate by adding the field and table names as resource
  strings in the application project.

 ---------------------------------------------------------------------------}

resourcestring
  { User terms }
  UT_NoAction = 'No action available'; {"No action availbale" message }
  UT_Search = 'Search'; { Field controller, Search command mode }
  UT_Insert = 'Insert'; { Field controller, Insert command mode }
  UT_Update = 'Update'; { Field controller, Update command mode }
  UT_SQL = 'SQL';  { Field controller, SQL command mode }
  UT_Add = 'Add';  { DataSearchGrid, Add update action }
  UT_Set = 'Set';  { DataSearchGrid, Set update action }
  UT_Replace = 'Replace'; { DataSearchGrid, Replace update action}
  UT_Delete = 'Delete'; { DataSearchGrid, Delete update action }
  UT_Action = 'Action';  { DataSearchGrid, Action header column }
  UT_Value = 'Value';  { DataSearchGrid, Value header column }
  UT_Image = 'Image'; { Report column header for the image column , if shown }
  UT_True = 'Yes'; { Boolean true}
  UT_False = 'No'; { Boolean false}
  UT_Null = 'Null'; {Null value}
  UT_Sort = 'Sort';  {Field controller sort mode }
  UT_Columns = 'Columns';  {Field controller columns order mode }
  UT_Source = 'Source'; {Source record in tree view move record }
  UT_Target = 'Target'; {Target record in tree view move record }
  UT_Recursive = 'Recursive';

  { Action bar button hints }
  UT_ABNextRecord = 'Next record';
  UT_ABPrevRecord = 'Previous record';
  UT_ABFirstRecord = 'First record';
  UT_ABLastRecord = 'Last record';
  UT_ABIncludeImages = 'Include image(s)';
  UT_ABInsert = 'Insert new record';
  UT_ABClear = 'Clear dataset';
  UT_ABUndo = 'Undo changes';
  UT_ABToggleViewMode = 'Toggle view mode';
  UT_ABEnterSearchMode = 'Enter search mode';
  UT_ABEnterUpdateMode = 'Enter update mode';
  UT_ABEnterInsertMode = 'Enter insert mode';
  UT_ABEnterColumnsMode = 'Enter columns mode';
  UT_ABEnterSortMode = 'Enter sort mode';
  UT_ABEnterSQLMode = 'Enter SQL mode';
  UT_ABExecute = 'Execute mode action';
  UT_ABFilter = 'Filter child records';
  UT_CBClearDACs = 'Clear values in data controls';
  UT_CBPopDACs = 'Populate data controls';
  UT_CBClearSingleDac = 'Clear value in active control';
  UT_CBShowAllControls = 'Show all data controls';
  UT_CBHideAllControls = 'Hide all data controls';
  UT_CBToggleDual = 'Toggle advanced and simple control mode';
  UT_ABSave = 'Save changes';
  UT_CBMoveUp = 'Move work field up';
  UT_CBMoveDown = 'Move work field down';
  UT_CBLogOp = 'Define logical operators';
  UT_CBAndOnly = 'AND only';
  UT_CBAndOrExclusively = 'AND / OR exclusively';
  UT_CBAndOrMixed = 'Combine AND / OR';
  UT_ABInsertImg = 'Insert with image';
  UT_ABInsertEmptyImg = 'Insert empty record with image';
  UT_ABUpdateImg = 'Update with image';

  { Progress messages }
   UT_PGSearch = 'Searching';
   UT_PGUpdate = 'Updating';
   UT_PGDelete = 'Deleting';
   UT_PGRemoveDeleted = 'Removing deleted records';
   UT_PGRecreateImages = 'Recreating images';
   UT_PGProcessTransLog = 'Processing trans log';
   UT_PGRecreateTextLobs = 'Recreating text objects';
   UT_PGAlterIndexSpace = 'Altering index space';
   UT_PGAlterRecordSize = 'Altering record size';
   UT_PGExtractIndexes = 'Extracting indexes';
   UT_PGBuildIndexes = 'Building indexes';
   UT_PGAlterTableSpace = 'Altering table space';
   UT_PGAlterRecordFreeSpace = 'Altering free space';
   UT_PGAlterRecordLayout = 'Altering record layout';
   UT_PGReadFile = 'Reading files';


  { Message bar terms }
   UT_MBReady = 'Ready';
   UT_MBReadyToSave = 'Ready to save';
   UT_MBReadyToUpdate = 'Ready to update';
   UT_MBRecordsInserted = 'record(s) inserted';
   UT_MBRecordsDeleted = 'record(s) deleted';
   UT_MBRecordsUpdated = 'record(s) updated';
   UT_MBRecordsChanged = 'Editing %s record(s)';
   UT_MBRecordsAdded = 'Inserting %s record(s)';
   UT_MBRecordsRemoved = 'Deleting %s record(s)';
   UT_MBRecordsFound = 'record(s) found';
   UT_MBRecords = 'record(s)';
   UT_MBRecordsSelected = 'selected';
   UT_MBRecordsSaved = 'record(s) saved';
   UT_MBRecordsAffected = '%s record(s) in %s table(s) affected';
   UT_MBRecordsNoneAffected = '0 records affected';

  { DB user errors }
  UE_NoActionAvailable = 'Action not available';
  Reserved = '';
  UE_SQLCommand = 'Could not resolve SQL type';
  UE_InvalidFieldName = 'Invalid or missing field name (%s)';
  UE_MissingValue = 'Missing value (%s)';
  UE_MissingOperator = 'Missing comparison operator';
  UE_IllegalValue = 'Illegal value (%s)';
  UE_IllegalMixOfEqualCo = 'Illegal mix of = and ==';
  UE_MissingQuote = 'Missing or incorrect number of quotes';
  UE_MissingParenthesis = 'Missing or misplaced parenthesis';
  UE_NotOpMisplaced = '"Not" operator misplaced';
  UE_NestedParentehis = 'Nested parenthesis not allowed';
  UE_FieldDupValues = 'Duplicate field values (%s)';
  UE_FieldNameExists = 'Field name or caption already exists';
  UE_UnsupportedFileFormat = 'Unsupported file format (%s)';
  UE_TextTooLong = 'Text too long (%s)';
  UE_IndexError1 = 'An error occured during indexing. Indexes may now be inconsistent. Your application will be terminated. ' +
    'Please run RebuildIndexes before proceeding. If the error continues, contact the vendor with the following message: %s';
  UE_FieldIsEmpty = 'Field is empty (%s)';
  UE_FieldDataTypeMismatch = 'Data type mismatch (%s)';
  UE_FileNotExists = 'File %s does not exist. Connection failed';
  UE_IndexError2 = 'An error occured during indexing. A rollback will be performed. If the error continues, ' +
     'contact the vendor with the following message: %s';
  UE_EntryExists = 'Unique constraint (%s) violated.';
  UE_FieldNullMixed = 'Cannot mix Null with regular values';
  UE_CtrlMismatch = 'Control is not legal or it might not fit the data type';
  UE_UpdateTempFields = 'Cannot update when dataset contains temporary fields';
  UE_SysFieldDefByName = 'System fields must be defined by name';
  UE_TooManyDecimals = 'Too many decimals (%s)';
  UE_RecMissingFields = 'Record has wrong number of fields';
  UE_NotAWorkCommand = 'Unrecognized command';
  UE_DatasetNil = 'Cannot set Dataset to nil';
  UE_FileExists = 'File already exists';
  UE_NotAViewCommand = 'Unrecognized command';
  UE_UpdateOperators = 'Update value operators cannot be mixed with regular values';
  UE_FieldTooManyValues = 'Too many values for this field (%s)';
  UE_CtrlNotFound = 'Control %s not registered in table %s';
  UE_InvalidTableName = 'Invalid table name (%s)';
  UE_QuotesNotNeeded = 'No quotes for this data type (%s)';
  UE_InvalidNumber = 'Invalid number (%s)';
  UE_AddSingleLineField = 'Cannot use the ADD keyword on a single line field. Use SET instead.';
  UE_MissingKeyword = 'Missing keyword';
  UE_NotReportField = 'Not a reportfield (%s)';
  UE_MissingFieldName = 'Missing field name';
  UE_QuoteError = 'QuoteError';
  UE_SyntaxError = 'Syntax error';
  UE_NotASelectedField = 'Order field %s is not a selected field';
  UE_MissingSelect = 'Missing SELECT clause';
  UE_NotANumberField = 'Not a number field';
  UE_MissingMandatoryField = 'Undefined mandatory field (%s)';
  UE_MissingWhere = 'Missing WHERE clause';
  UE_MissingCommand = 'Missing or invalid command / target';
  UE_InappropriateWhere = 'Condition or CONNECT BY not allowed when target is Dataset';
  UE_ImageLinkNotFound = 'Image link not found in record';
  UE_MissingExpression = 'Missing expression';
  UE_MissingLinkField = 'Missing link field';
  UE_InvalidSource = 'Invalid or incompatible source';
  UE_CopyNoImage = 'Record contains no image';
  UE_ClipBrdNoLink = 'Clipboard does not contain an image link';
  UE_ClipBrdNoImage = 'Clipboard does not contain an image';
  UE_DatasetReadOnly = 'Dataset is read only';
  UE_UndefinedLinkRecord = 'Undefined parent record';
  UE_SaveMasterRecord = 'Parent record referenced must be saved before connecting the child';
  UE_DupIdentifiers = 'Identifier(s) already exist(s) in dataset';
  UE_VarError = '%s'; {Variable error message, coming from OnEvaluateRecord}
  UE_NoConnection = 'No connection';
  UE_ConnectionsActive = 'Cannot perform this operation when other connections are active';
  UE_MissingImage = 'Missing image(s)';
  UE_ReadOnly = 'Data is read only (%s)';
  UE_InvalidDate = 'Invalid date (%s)';
  UE_InvalidBoolean = 'Not a boolean value';
  UE_SearchGridDataCtrl = 'Search grids cannot be data controls';
  UE_SaveImage = 'Image must be saved before you can add it to the image table';
  UE_OrderBySiblings = 'ORDER BY SIBLINGS can only be used with tree view tables';
  UE_StartWith = 'START WITH can only be used with tree view tables';
  UE_Circular = 'Circular reference!';
  UE_TVMoveRecord = '%s record not found in dataset';
  UE_SiblingOutOfBounds = 'Sibling index out of bounds';
  UE_SiblingsNotComplete = 'Sibling list not complete in dataset selection';
  UE_MissingUpdateSpec = 'Missing update specifier(%s)';
  UE_FreeTableMaster = 'Stand alone tables cannot have a master source';
  UE_TVReference = 'Cannot link to a deleted or undefined node.';
  UE_TagNameNotFound = 'Tag name %s not found';
  UE_DeleteMandatory = 'Cannot delete a mandatory field (%s)';
  UE_TransError = 'An error occurred ("%s"). Operation aborted.';
  UE_OperatorMismatch = 'Operator %s cannot be applied for this data type / operation';
  UE_Formula = 'An error occurred during calculation. Check your formula';
  UE_CalcDatatype = 'Only text, number and date types allowed'; {Wrong user calc data type }
  UE_MissingFormula = 'Missing formula'; {Mising formula in user calc}
  UE_CalcActiveTable = '%s is not the active table'; {User trying to set formula on table that is not the active one }
  UE_RelConstraint = 'This table only supports single records or single child relationships'; { Too many records }
  UE_MissingConnectBy = 'Missing CONNECT BY'; {Insert sql misses mandatory connect by}
  UE_InvalidDBFileName = 'Invalid database file name';
  UE_LoadDatabase = 'Could not load database';
  UE_ParentJoin = 'Cannot join a parent table (%s).';
  UE_TreeViewJoin = 'Cannot join from a treeview table';
  UE_TempColumns = 'Column handling is not available when Dataset contains temporary or joined fields';
  UE_ForeignOperands = 'Cannot use foreign fields as operands (%s)';
  UE_FieldReadOnly = 'Field is read only(%s)';
  UE_IllegalMVFOperator = 'The %s operator can only be applied on multi value fields';
  UE_ImageNotAvailable = 'Image not available';
  UE_UnresolvedTableKey = 'Could not decide table reference.';
  UE_DatasetIsEmpty = 'Dataset is empty';
  UE_MandatoryValue = 'Missing mandatory value (%s)';
  UE_TransActive = 'A transaction is already running';


{ Developer errors }
DE_DBNotIdle =  'Database is not yet fully loaded. Use the OnLoaded event handler of the table manager ' +
 ' when connecting on start up';
DE_InternetAccess = 'Error accessing Internet';
DE_FixedNotStatic = 'Fixed string must have a static recordspace';
DE_ChangeSorted = 'Cannot change items in a sorted file';
DE_SpaceOverflow = 'String length exceeds length of fixed data space: %s';
DE_PropOnClosedTable = 'Cannot set this property on a closed table';
DE_TXTNotIdentifier = 'Text or Resource fields cannot be identifiers';
DE_DataTypeMaxLen = 'Max length too long for this datatype';
DE_IdentSingleLine = 'Identifier must be a single line field';
DE_InvalidCndOp = 'Invalid conditional operator';
DE_UnknownPID = 'Unknown PID identifier';
DE_NotCalcField = 'Not a calulated field';
DE_NotAParentTable = 'Table %s is not a parented table';{Used with TILSQL and filter params}
DE_CreateIndexes = 'Internal error creating indexes';
DE_DupTableName = 'Duplicate table or alias name';
DE_RegKey = 'Could not create registry key %s';
DE_MissingTableName = 'Missing table name or table not assigned';
DE_OutOfBounds = 'Index out of bounds (%s)';
DE_RecordNotFound = 'The requested record at index %s was not found';
DE_InitTrans = 'Could not initialize transaction';
DE_NotInScrollbox = 'Parent control is not a scrollbox';
DE_TableSpaceTooSmall = 'Table space cannot be less than 10KB';
DE_ClearTable = 'Cannot clear this table as it is referenced by records in table %s';
DE_MethodMastertable = 'This method can only be called from the master table'; // Obs!
DE_NoTableNameLoc = 'Missing table name or location spec. Operation aborted.';
DE_AlterTransActive = 'Cannot alter table when a transaction is active';
DE_DatasetMismatch = 'Dataset is not compatible with table';
DE_DropIndexIdentifier = 'Cannot drop index on an identifier field (%s)'; // Obs
DE_NotAnIndexedField = 'Not an indexed field'; // Obs
DE_MissingMeta = 'Missing meta information file (%s).';
DE_NoBackup = 'Could not create backup of %s (%s)';
DE_InvalidIndexSpace = 'IndexSpace must be a number between 10 and 255';
DE_CmdOptionNotFound = 'Option not found or registered with this command';
DE_UndefinedMasterTable = 'Undefined master table';
DE_FieldNameRequired = 'Set FieldName before assigning this property';
DE_CtrlAllowed = 'Only %s allowed';
DE_DupController = 'Only one controller object allowed for the database';
DE_CommandsMinimum = 'Commands must contain the Search or the SQL command as a minimum.';
DE_ControlMix = 'You cannot mix own controls with controls generated by the controller';
DE_InvalidCommandName = 'Unknown command name (%s)';
DE_TableNotInDB = 'Table %s not registered in database. To register a table run CreateTable. Connection failed';
DE_MissingMasterSource = 'Missing MasterSource';
DE_ButtonAvailable = 'Button not availbale. Consider turning off PreviewWorkstyle';
DE_CtrlDataTypeMismatch = 'Control type and data type do not go together';
DE_TableOutOfPath = 'Table not in search path (%s)';
DE_LookupNotRegistered = 'Lookup value %s not registered';
DE_LookupDelete = 'Deleting this record violates database integrity as the value of the field %s is referenced by another table';
DE_ReservedName = 'Reserved name, find another';
DE_TreeViewSysField = 'This field name can only be used with treeview tables.';
DE_MissingRootTable = 'Missing root table';
DE_FreeTables = 'Free tables cannot be part of the table hieararchy';
DE_RootDetail = 'Root table cannot have a master source';
DE_InvalidChar = 'Invalid character in name';
DE_MissingChild = 'Missing child table';
DE_MissingParent = 'Missing parent table';
DE_ParentEqChild = 'Parent and child table cannot be the same';
DE_LibNotCreated = 'Database not created';
DE_SysFieldLookup = 'System fields cannot have lookup tables';
DE_InvalidLookupRef = 'Invalid lookup reference';
DE_TreeViewChildTables = 'Treeview tables cannot be child tables';
DE_ManagerNotFound = 'Table manager not found';
DE_SearchFieldOrder = 'Search field order list is missing or incomplete';
DE_TreeViewLabel = 'This field is not suitable as tree view label';
DE_UnKnownTableClass = 'Unknown table type (%s)';
DE_DupButtonType = 'Duplicate button type';
DE_MasterTblNotRoot = 'Master table must be the root of the hierarchy';
DE_CmdNotEnlisted = 'Command not enlisted';
DE_CreateConnected = 'When creating a table you must run app with no connection';
DE_TableNoFields = 'Field list of table %s is incomplete. Note that TableManager must be assigned before creating the table';
DE_MissingDBName = 'Missing database name';
DE_DBInvalid = '%s: %s has changed. Table tree data is no longer valid. To continue you must clear the table tree (run TableManager.ClearTableTree) or rechange table structure.'; { Not used}
DE_MissingInsertFields = 'Missing insert field (%s)';
DE_InvalidMetaFile = 'Invalid meta file type';
DE_InvalidDir = 'Directory does not exist (%s)';
DE_UpdateZoomed = 'Cannot update a zoomed image';
DE_ParentResolve = 'Parent cannot be resolved';
DE_SiblingsIncomplete = 'Sibling list is not complete in Dataset';
DE_ImageSourceRecord = 'Source record must be fetched from a regular TILDataTable';
DE_DBConstruct = 'Source database is still under construction';
DE_OptDefaultNeeded = 'The ovDefault option is manadatory when search model is smAndOrMoxed';
DE_OptSearchAndNeeded = 'The ovSearchAnd option is mandatory when search model is smAnAligned';
DE_LooTableNotDefined1 = 'Note! Lookup properties for %s is incomplete. You must correct this else runtime errors will occur.'; { Check onloaded }
DE_LooTableNotDefined2 = 'Note! Lookup properties for %s is incomplete. Connection failed.'; { Check onloaded }
DE_MissingSourceTable = 'SourceTable must be assigned for this operation.'; { When setting controltype in TILPanel }
DE_TableNotAssigned = 'Table not assigned'; { LogOpButton }
DE_DacNotRegistered = '%s is not a registered %s control'; { LogOp button }
DE_LooFieldType = 'Lookup fields require the %s type'; {Wrong Lookup field data / field type }
DE_AutoIncTableEmpty = 'Cannot change this value when table is not empty';
DE_ValuePickList = 'Value %s not in picklist'; { Initial value not in picklist; looPickValues }
DE_LooUnique = 'Records in the LookupTable must be mandatory and uniquely identified with the Unique or Identifier flag'; { Lookup field maust be unique }
DE_LooTableNotDefined = 'LookupTable must be assigned before setting LookupField'; { Check on SetLookupfield }
DE_LooFieldRef = 'The field in the lookup table cannot be a referenced lookup field.'; { Cannot look up refrenced fields }
DE_LooIdent = 'The field in the lookup table cannot be part of a compound identifier key'; {Look up field is part of a compund identifdier }
DE_ViewerTable = 'Another viewer is already connected to this data source'; { Another viewer is using the source }
DE_NotUserCalcField = '%s is not a user calculated field'; {Trying to address a user calc field that is not defined as user calc }
DE_ConnectByLiteral = 'Connect must be a conditional expression, not a single number'; { Literals not allowed in connect by inserts }
DE_ConnectByRoot = 'CONNECT BY cannot be applied on a root table or record'; { Cannot use connect by on a root table or record }
DE_LogOpBtnType = 'The %s type can only be used with the %s search model'; {Illegal log op button type }
DE_LookupSelf = 'LookupTable cannot be this table'; {Trying to set LookupTable to the table that contains the loo field}
DE_LooKeyDataTable = 'The looTableKey lookup type requires LookupTable to be a data table in the table tree'; { The looTableKey type must bee a data table in the table tree }
DE_LooKeyPath = 'The looTableKey lookup type requires LookupTable to be in a different path'; { The looTableKey type requires table to be in different paths }
DE_LooKeyNameOrNumber = 'The looTableKey lookup type requires data type to be either dtString og dtNumber'; {The looTableKey must be name or string or number }
DE_LooSearchField = 'This property is only relevant for the looTableKey lookup type'; {Trying to set LookupSearchField when lootype is not looTableKey}
DE_PropLock = 'Cannot change this property due to limitations set by other properties'; { Field property cannot be set because of conflict with other properties}
DE_FreeSpace = 'RecordFreeSpace must be a number between 0 and 255'; { Invalid record space value }
DE_InvalidTableKey = 'Lookup table key %s does not exist in lookup table'; {Lookup table key does not exist in lookup table}
DE_MissingVALUES = 'Missing VALUES key word'; { Missing VALUES key word in insert sql}
DE_MissingLookupTable = 'Referencing table (Image- or LookupTable) not found or not assigned to TableManager.'; {Not in project or not assigned to TM when used for sharing purposes}
DE_TBLErrorState = 'Table %s is in an erronious state. Use the administration app to detect and correct the error. Connection failed.';
DE_RecordSizeTooSmall = 'Record size too small'; // OBS
DE_DupFieldName = 'Duplicate field name';
DE_InvalidSortField = 'Sort field %s is invalid or not in field list';
DE_IllegalSortfield = 'Field %s cannot be used as as sort field';
DE_ImgTableNotAssigned = 'SharedImages of table manager must be assigned when sharing images';
DE_TableNotEmpty = 'Cannot execute this operation on a none empty table';
DE_DBLoaded = 'Cannot create DB when a database is already loaded';
DE_DBNotLoaded = 'No database loaded!';
DE_MissingMetaField = 'Could not locate field %s in meta description';
DE_ResSingleLine = 'Cannot add multiple resources when mode is SingleLine';
DE_MissingResElement = 'Missing resource element';
DE_ResSearchControl = 'Resource lists cannot be used as search controls';
DE_MissingResName = 'Missing resource name';
DE_MissingResAddress = 'Missing resource address';
DE_DupDBName = 'This database name used by another database';
DE_ImageSpecifier = 'Image specifier must be at end of field list';
DE_PanelCtrls = 'Cannot use same control for data and search in control panels';
DE_StrImageTag = 'Only string fields can be used as image tags';
DE_DataTableImageTag = 'Only data tables can have image tags';
DE_IndexingConnected = 'You must be connected to database to perform indexing';
DE_NotDesignTime = 'This operation can only be executed run time.';
DE_TableOperation = 'Operation not applicable for this table type';
DE_MissingViewTable = 'Source table for view %s is undefined';
DE_IllegalCommandMode = 'Command mode must be either cmSearch or cmUpdate';
DE_CircularTrigger = 'Circular trigger call';
DE_TableDLL = 'This table type cannot be used with ILib.DLL';
DE_NoUpdateWhere = 'WHERE condition inappropriate here';
DE_ViewerNoHandler = 'Table %s does not have an associated Data object. Add a TILData component and set its Table property';
DE_DataDupTable = 'Another Data object (%s) is already using this table.';
DE_DVNoSourceTable = 'Data source has no table asignment';


const
  E_ErrTextCount = 115;
  T_TermCount = 83;

  T_NoAction = 0; {"No action availbale" message }
  T_Search = 1; { Field controller, Search command mode }
  T_Insert = 2; { Field controller, Insert command mode }
  T_Update = 3; { Field controller, Update command mode }
  T_SQL = 4;  { Field controller, SQL command mode }
  T_Add = 5;  { DataSearchGrid, Add update action }
  T_Set = 6;  { DataSearchGrid, Set update action }
  T_Replace = 7; { DataSearchGrid, Replace update action}
  T_Delete = 8; { DataSearchGrid, Delete update action }
  T_Action = 9;  { DataSearchGrid, Action header column }
  T_Value = 10;  { DataSearchGrid, Value header column }
  T_Image = 11; { Report column header for the image column , if shown }
  T_True = 12; { Boolean true}
  T_False = 13; { Boolean false}
  T_Null = 78; {Null value}
  T_Sort = 14;  {Field controller sort mode }
  T_Columns = 15;  {Field controller columns order mode }
  T_Source = 16; {Source record in tree view move record }
  T_Target = 17; {Target record in tree view move record }
  T_Recursive = 18;

  { Action bar button hints }
  T_ABNextRecord = 19;
  T_ABPrevRecord = 20;
  T_ABFirstRecord = 21;
  T_ABLastRecord = 22;
  T_ABIncludeImages = 23;
  T_ABInsert = 24;
  T_ABClear = 25;
  T_ABUndo = 26;
  T_ABToggleViewMode = 27;
  T_ABEnterSearchMode = 28;
  T_ABEnterUpdateMode = 29;
  T_ABEnterInsertMode = 30;
  T_ABEnterColumnsMode = 31;
  T_ABEnterSortMode = 32;
  T_ABEnterSQLMode = 33;
  T_ABExecute = 34;
  T_CBClearDACs = 35;
  T_CBPopDACs = 36;
  T_CBClearSingleDac = 77;
  T_CBShowAllControls = 37;
  T_CBHideAllControls = 38;
  T_CBToggleDual = 39;
  T_ABSave = 40;
  T_CBMoveUp = 41;
  T_CBMoveDown = 42;
  T_CBLogOp = 43;
  T_CBAndOnly = 44;
  T_CBAndOrExclusively = 45;
  T_CBAndOrMixed = 46;
  T_ABInsertImg = 47;
  T_ABInsertEmptyImg = 48;
  T_ABUpdateImg = 49;
  T_ABFilter = 50;

  { Progress messages}
  T_PGSearch = 51;
  T_PGUpdate = 52;
  T_PGDelete = 53;
  T_PGRemoveDeleted = 54;
  T_PGRecreateImages = 55;
  T_PGProcessTransLog = 56;
  T_PGRecreateTextLobs = 57;
  T_PGAlterIndexSpace = 58;
  T_PGAlterRecordSize = 59;
  T_PGExtractIndexes = 60;
  T_PGBuildIndexes = 61;
  T_PGAlterTableSpace = 62;
  T_PGAlterRecordFreeSpace = 63;
  T_PGAlterRecordLayout = 64;
  T_PGReadFile = 65;

  { Message bar terms }
  T_MBReady = 66;
  T_MBReadyToSave = 67;
  T_MBReadyToUpdate = 68;
  T_MBRecordsInserted = 69;
  T_MBRecordsDeleted = 70;
  T_MBRecordsUpdated = 71;
  T_MBRecordsChanged = 72;
  T_MBRecordsFound = 73;
  T_MBRecords = 74;
  T_MBRecordsSelected = 75;
  T_MBRecordsSaved = 76;
  T_MBRecordsAdded = 79;
  T_MBRecordsRemoved = 80;
  T_MBRecordsAffected = 81;
  T_MBRecordsNoneAffected = 82;


  T_Terms : array[0.. T_TermCount-1] of string =
 {0} (UT_NoAction ,
 {1} UT_Search ,
 {2} UT_Insert ,
 {3} UT_Update ,
 {4} UT_SQL ,
 {5} UT_Add ,
 {6} UT_Set ,
 {7} UT_Replace ,
 {8} UT_Delete ,
 {9} UT_Action ,
 {10} UT_Value ,
 {11} UT_Image ,
 {12} UT_True ,
 {13} UT_False ,
 {14} UT_Sort ,
 {15} UT_Columns ,
 {16} UT_Source ,
 {17} UT_Target ,
 {18} UT_Recursive,
 {19} UT_ABNextRecord,
 {20} UT_ABPrevRecord,
 {21} UT_ABFirstRecord,
 {22} UT_ABLastRecord,
 {23} UT_ABIncludeImages,
 {24} UT_ABInsert,
 {25} UT_ABClear,
 {26} UT_ABUndo,
 {27} UT_ABToggleViewMode,
 {28} UT_ABEnterSearchMode,
 {29} UT_ABEnterUpdateMode,
 {30} UT_ABEnterInsertMode,
 {31} UT_ABEnterColumnsMode,
 {32} UT_ABEnterSortMode,
 {33} UT_ABEnterSQLMode,
 {34} UT_ABExecute,
 {35} UT_CBClearDACs,
 {36} UT_CBPopDACs,
 {37} UT_CBShowAllControls,
 {38} UT_CBHideAllControls,
 {39} UT_CBToggleDual,
 {40} UT_ABSave,
 {41} UT_CBMoveUp,
 {42} UT_CBMoveDown,
 {43} UT_CBLogOp,
 {44} UT_CBAndOnly,
 {45} UT_CBAndOrExclusively,
 {46} UT_CBAndOrMixed,
 {47} UT_ABInsertImg,
 {48} UT_ABInsertEmptyImg,
 {49} UT_ABUpdateImg,
 {50} UT_ABFilter,
 {51} UT_PGSearch,
 {52} UT_PGUpdate,
 {53} UT_PGDelete,
 {54} UT_PGRemoveDeleted,
 {55} UT_PGRecreateImages,
 {56} UT_PGProcessTransLog,
 {57} UT_PGRecreateTextLobs,
 {58} UT_PGAlterIndexSpace,
 {59} UT_PGAlterRecordSize,
 {60} UT_PGExtractIndexes,
 {61} UT_PGBuildIndexes,
 {62} UT_PGAlterTableSpace,
 {63} UT_PGAlterRecordFreeSpace,
 {64} UT_PGAlterRecordLayout,
 {65} UT_PGReadFile,
 {66} UT_MBReady,
 {67} UT_MBReadyToSave,
 {68} UT_MBReadyToUpdate,
 {69} UT_MBRecordsInserted,
 {70} UT_MBRecordsDeleted,
 {71} UT_MBRecordsUpdated,
 {72} UT_MBRecordsChanged,
 {73} UT_MBRecordsFound,
 {74} UT_MBRecords,
 {75} UT_MBRecordsSelected,
 {76} UT_MBRecordsSaved,
 {77} UT_CBClearSingleDAC,
 {78} UT_Null,
 {79} UT_MBRecordsAdded,
 {80} UT_MBRecordsRemoved,
 {81} UT_MBRecordsAffected,
 {82} UT_MBRecordsNoneAffected
 );

  { DB user errors }
  E_SQLCommand = 1;
  E_InvalidFieldName = 2;
  E_MissingValue = 3;
  E_MissingOperator = 4;
  E_IllegalValue = 5;
  E_IllegalMixOfEqualCo = 6;
  E_MissingQuote = 7;
  E_MissingParenthesis = 8;
  E_NotOpMisplaced = 9;
  E_NestedParentehis = 10;
  E_FieldDupValues = 11;
  E_FieldNameExists = 12;
  E_UnsupportedFileFormat = 13;
  E_TextTooLong = 14;
  E_IndexError1 = 15; // not used
  E_FieldIsEmpty = 16;
  E_FieldDataTypeMismatch = 17;
  E_FileNotExists = 18;
  E_IndexError2 = 19; // not used
  E_EntryExists = 20;
  E_MandatoryValue = 21;
  E_FieldNullMixed = 22;
  E_CtrlMismatch = 23;
  E_UpdateTempFields = 24;
  E_UnresolvedTableKey = 25;
  E_SysFieldDefByName = 26; // Unused
  E_TooManyDecimals = 27;
  E_DatasetIsEmpty = 28;
  E_AlterTransActive = 29; // Unused
  {30 NOT USED}
  E_RecMissingFields = 31;
  E_NotAWorkCommand = 32;
  E_DatasetNil = 33;
  {34 NOT USED}
  E_FileExists = 35;
  E_NotAViewCommand = 36;
  E_UpdateOperators = 37;
  E_FieldTooManyValues = 38;
  {39 NOT USED}
  E_CtrlNotFound = 40;
  E_InvalidTableName = 41;
  {42 NOT USED}
  E_QuotesNotNeeded = 43;
  E_InvalidNumber = 44;
  E_AddSingleLineField = 45;
  E_MissingKeyword = 46;
  {47 NOT USED}
  E_NotReportField = 48;
  E_MissingFieldName = 49;
  E_QuoteError = 50;
  E_SyntaxError = 51;
  E_NotASelectedField = 52;
  E_MissingSelect = 53;
  E_NotANumberField = 54;
  E_MissingMandatoryField = 55;
  E_MissingWhere = 56;
  E_MissingCommand = 57;
  E_InappropriateWhere = 58;
  E_ImageLinkNotFound = 59;
  E_MissingExpression = 60;
  E_MissingLinkField = 61;
  E_InvalidSource = 62;
  E_CopyNoImage = 63;
  E_ClipBrdNoLink = 64;
  E_ClipBrdNoImage = 65;
  E_DatasetReadOnly = 66;
  E_UndefinedLinkRecord = 67;
  E_SaveMasterRecord = 68;
  E_DupIdentifiers = 69;
  E_VarError = 70; // Not used
  E_NoConnection = 71;
  E_ConnectionsActive = 72;
  E_MissingImage = 73;
  E_ReadOnly = 74;
  {75 NOT USED}
  {76 NOT USED}
  E_InvalidDate = 77;
  E_InvalidBoolean = 78;
  E_SearchGridDataCtrl = 79;
  E_SaveImage = 80;
  E_OrderBySiblings = 81;
  E_StartWith = 82;
  {83 NOT USED}
  E_Circular = 84;
  E_TVMoveRecord = 85;
  E_SiblingOutOfBounds = 86;
  E_SiblingsNotComplete = 87;
  E_MissingUpdateSpec = 88;
  E_FreeTableMaster = 89;
  E_TVReference = 90;
  {91 NOT USED}
  E_TagNameNotFound = 92;
  E_DeleteMandatory = 93;
  E_TransError = 94;
  E_OperatorMismatch = 95;
  E_Formula = 96;
  E_CalcDatatype = 97; {Wrong user calc data type }
  E_MissingFormula = 98; {Mising formula in user calc}
  E_CalcActiveTable = 99; {User trying to set formula on table that is not the active one }
  E_RelConstraint = 100; { Too many records }
  E_MissingConnectBy = 101; {Insert sql misses mandatory connect by}
  E_InvalidDBFileName = 102;
  E_LoadDatabase = 103;
  E_ParentJoin = 104;
  E_TreeViewJoin = 105;
  E_TempColumns = 106;
  E_ForeignOperands = 107;
  E_FieldReadOnly = 108;
  E_IllegalMVFOperator = 109;
  E_ImageNotAvailable = 110;


E_ErrorText : array[1.. E_ErrTextCount] of string =
{1} (UE_SQLCommand ,
{2} UE_InvalidFieldName ,
{3} UE_MissingValue ,
{4} UE_MissingOperator ,
{5} UE_IllegalValue ,
{6} UE_IllegalMixOfEqualCo ,
{7} UE_MissingQuote ,
{8} UE_MissingParenthesis ,
{9} UE_NotOpMisplaced ,
{10} UE_NestedParentehis ,
{11} UE_FieldDupValues ,
{12} UE_FieldNameExists ,
{13} UE_UnsupportedFileFormat ,
{14} UE_TextTooLong ,
{15} UE_IndexError1 ,
{16} UE_FieldIsEmpty ,
{17} UE_FieldDataTypeMismatch ,
{18} UE_FileNotExists ,
{19} UE_IndexError2 ,
{20} UE_EntryExists ,
{21} UE_MandatoryValue ,
{22} UE_FieldNullMixed ,
{23} UE_CtrlMismatch ,
{24} UE_UpdateTempFields ,
{25} UE_UnresolvedTableKey ,
{26} UE_SysFieldDefByName ,
{27} UE_TooManyDecimals ,
{28} UE_DatasetIsEmpty,
{29} UE_TransActive ,
{30} Reserved ,
{31} UE_RecMissingFields ,
{32} UE_NotAWorkCommand ,
{33} UE_DatasetNil ,
{34} Reserved ,
{35} UE_FileExists ,
{36} UE_NotAViewCommand ,
{37} UE_UpdateOperators ,
{38} UE_FieldTooManyValues ,
{39} Reserved ,
{40} UE_CtrlNotFound ,
{41} UE_InvalidTableName ,
{42} Reserved ,
{43} UE_QuotesNotNeeded ,
{44} UE_InvalidNumber ,
{45} UE_AddSingleLineField ,
{46} UE_MissingKeyword ,
{47} Reserved,
{48} UE_NotReportField ,
{49} UE_MissingFieldName ,
{50} UE_QuoteError ,
{51} UE_SyntaxError ,
{52} UE_NotASelectedField ,
{53} UE_MissingSelect ,
{54} UE_NotANumberField ,
{55} UE_MissingMandatoryField ,
{56} UE_MissingWhere ,
{57} UE_MissingCommand ,
{58} UE_InappropriateWhere ,
{59} UE_ImageLinkNotFound ,
{60} UE_MissingExpression ,
{61} UE_MissingLinkField ,
{62} UE_InvalidSource ,
{63} UE_CopyNoImage ,
{64} UE_ClipBrdNoLink ,
{65} UE_ClipBrdNoImage ,
{66} UE_DatasetReadOnly ,
{67} UE_UndefinedLinkRecord ,
{68} UE_SaveMasterRecord ,
{69} UE_DupIdentifiers ,
{70} UE_VarError, { Variable error message }
{71} UE_NoConnection ,
{72} UE_ConnectionsActive ,
{73} UE_MissingImage , { Controller: OnGetImage }
{74} UE_ReadOnly ,
{75} Reserved ,
{76} Reserved,
{77} UE_InvalidDate ,
{78} UE_InvalidBoolean ,
{79} UE_SearchGridDataCtrl ,
{80} UE_SaveImage ,
{81} UE_OrderBySiblings ,
{82} UE_StartWith ,
{83} Reserved ,
{84} UE_Circular ,
{85} UE_TVMoveRecord ,
{86} UE_SiblingOutOfBounds ,
{87} UE_SiblingsNotComplete ,
{88} UE_MissingUpdateSpec , {Set, Replace, Delete or Add }
{89} UE_FreeTableMaster ,
{90} UE_TVReference ,
{91} Reserved ,
{92} UE_TagNameNotFound ,
{93} UE_DeleteMandatory ,
{94} UE_TransError ,
{95} UE_OperatorMismatch ,
{96} UE_Formula ,
{97} UE_CalcDatatype ,
{98} UE_MissingFormula ,
{99} UE_CalcActiveTable ,
{100} UE_RelConstraint ,
{101} UE_MissingConnectBy ,
{102} UE_InvalidDBFileName ,
{103} UE_LoadDatabase ,
{104} UE_ParentJoin,
{105} UE_TreeViewJoin,
{106} UE_TempColumns,
{107} UE_ForeignOperands,
{108} UE_FieldReadOnly,
{109} UE_IllegalMVFOperator,
{110} UE_ImageNotAvailable,
{111} '',
{112} '',
{113} '',
{114} '',
{115} ''
);

E_InternetAccess = 200;
E_FixedNotStatic = 201;
E_ChangeSorted = 202;
E_SpaceOverflow = 203;
E_PropOnClosedTable = 204;
E_TXTNotIdentifier = 205;
E_DataTypeMaxLen = 206;
E_IdentSingleLine = 207;
E_StrImageTag = 208;
E_DataTableImageTag = 209;
E_UnknownPID = 210;
E_NotCalcField = 211;
E_InvalidCndOp = 212;
E_CreateIndexes = 213;
E_DupTableName = 214;
E_RegKey = 215;
E_MissingTableName = 216;
E_OutOfBounds = 217;
E_InitTrans = 218;
E_NotInScrollbox = 219;
E_TableSpaceTooSmall = 220;
E_ClearTable = 221;
E_MethodMastertable = 222; // Obs!
E_NoTableNameLoc = 223;
E_TransActive = 224;
E_DatasetMismatch = 225;
E_DropIndexIdentifier = 226; // Obs
E_NotAnIndexedField = 227; // Obs
E_MissingMeta = 228;
E_IndexingConnected = 229;
E_NotDesignTime = 230;
E_TableOperation = 231;
E_MissingViewTable = 232;
E_IllegalCommandMode = 233;
E_RecordNotFound = 234;
E_NoUpdateWhere = 235;
E_NoBackup = 236;
E_InvalidIndexSpace = 237;
E_ViewerNoHandler = 238;
E_CmdOptionNotFound = 239;
E_UndefinedMasterTable = 240;
E_DataDupTable = 241;
E_DVNoSourceTable = 242;
E_FieldNameRequired = 243;
E_CtrlAllowed = 244;
E_DupController = 245;
E_CommandsMinimum = 246;
E_ControlMix = 247;
E_InvalidCommandName = 248;
E_DBNotIdle = 249;
{249 NOT USED}
{250 NOT USED}
E_TableNotInDB = 251;
{252 NOT USED}
E_MissingMasterSource = 253;
{254 NOT USED}
{255 NOT USED}
E_ButtonAvailable = 256;
{257 NOT USED}
{258 NOT USED}
E_CtrlDataTypeMismatch = 259;
E_TableOutOfPath = 260;
E_LookupNotRegistered = 261;
E_LookupDelete = 262;
E_ReservedName = 263;
E_TreeViewSysField = 264;
{265 NOT USED}
E_MissingRootTable = 266;
E_FreeTables = 267;
E_RootDetail = 268;
E_InvalidChar = 269;
E_NotAParentTable = 270;
E_MissingChild = 271;
E_MissingParent = 272;
E_ParentEqChild = 273;
E_LibNotCreated = 274;
E_CircularTrigger = 275;
E_SysFieldLookup = 276;
E_InvalidLookupRef = 277;
E_TreeViewChildTables = 278;
E_ManagerNotFound = 279;
E_SearchFieldOrder = 280;
E_TreeViewLabel = 281;
E_UnKnownTableClass = 282;
E_DupButtonType = 283;
E_MasterTblNotRoot = 284;
E_CmdNotEnlisted = 285;
E_CreateConnected = 286;
{287 NOT USED}
E_TableNoFields = 288;
{289 NOT USED}
{290 NOT USED}
E_MissingDBName = 291;
{292 NOT USED}
{293 NOT USED}
{294 NOT USED}
E_MissingInsertFields = 295;
E_InvalidMetaFile = 296;
{297 NOT USED}
E_InvalidDir = 298;
E_PanelCtrls = 299;
{300 NOT USED}
E_UpdateZoomed = 301;
E_ParentResolve = 302;
{303 NOT USED}
E_SiblingsIncomplete = 304;
E_ImageSourceRecord = 305;
{306 NOT USED}
E_DBConstruct = 307;
E_OptDefaultNeeded = 308;
E_OptSearchAndNeeded = 309;
E_LooTableNotDefined1 = 310; { Check onloaded }
E_LooTableNotDefined2 = 311; { Check onloaded }
E_MissingSourceTable = 312; { When setting controltype in TILPanel }
E_TableNotAssigned = 313; { LogOpButton }
E_DacNotRegistered = 314; { LogOp button }
E_LooFieldType = 315; {Wrong Lookup field data / field type }
E_AutoIncTableEmpty = 316;
E_ValuePickList = 317; { Initial value not in picklist; looPickValues }
E_LooUnique = 318; { Lookup field maust be unique }
E_LooTableNotDefined = 319; { Check on SetLookupfield }
E_LooFieldRef = 320; { Cannot look up refrenced fields }
E_LooIdent = 321; {Look up field is part of a compund identifdier }
E_ViewerTable = 322; { Another viewer is using the table }
E_NotUserCalcField = 323; {Trying to address a user calc field that is not defined as user calc }
E_ConnectByLiteral = 324; { Literals not allowed in connect by inserts }
E_ConnectByRoot = 325; { Cannot use connect by on a root table or record }
{326 NOT USED}
{327 NOT USED}
E_LogOpBtnType = 328; {Illegal log op button type }
E_LookupSelf = 329; {Trying to set LookupTable to the table that contains the loo field}
E_LooKeyDataTable = 330; { The looTableKey type must bee a data table in the table tree }
E_LooKeyPath = 331; { The looTableKey type requires table to be in different paths }
E_LooKeyNameOrNumber = 332; {The looTableKey must be name or string or number }
E_LooSearchField = 333; {Trying to set LookupSearchField when lootype is not looTableKey}
E_PropLock = 334; { Field property cannot be set because of conflict with other properties}
E_FreeSpace = 335; { Invalid record space value }
E_InvalidTableKey = 336; {Lookup table key does not exist in lookup table}
E_MissingVALUES = 337; { Missing VALUES key word in insert sql}
E_MissingLookupTable = 338; {Not in project or not assigned to TM when used for sharing purposes}
E_TBLErrorState = 339;
E_RecordSizeTooSmall = 340; // OBS
E_DupFieldName = 341;
E_InvalidSortField = 342;
E_IllegalSortfield = 343;
E_ImgTableNotAssigned = 344;
E_TableNotEmpty = 345;
E_DBLoaded = 346;
E_DBNotLoaded = 347;
E_MissingMetaField = 348;
E_TableDLL = 349;
E_ResSingleLine = 350;
E_MissingResElement = 351;
E_ResSearchControl = 352;
E_MissingResName = 353;
E_MissingResAddress = 354;
E_DupDBName = 355;
E_ImageSpecifier = 356;

E_InternalsCount = 159;
E_Internals : array[200..200 + E_InternalsCount-1] of string =
{200} (DE_InternetAccess ,
{201} DE_FixedNotStatic ,
{202} DE_ChangeSorted ,
{203} DE_SpaceOverflow ,
{204} DE_PropOnClosedTable ,
{205} DE_TXTNotIdentifier ,
{206} DE_DataTypeMaxLen ,
{207} DE_IdentSingleLine ,
{208} DE_StrImageTag,
{209} DE_DataTableImageTag ,
{210} DE_UnknownPID ,
{211} DE_NotCalcField ,
{212} DE_InvalidCndOp ,
{213} DE_CreateIndexes ,
{214} DE_DupTableName ,
{215} DE_RegKey ,
{216} DE_MissingTableName ,
{217} DE_OutOfBounds ,
{218} DE_InitTrans ,
{219} DE_NotInScrollbox ,
{220} DE_TableSpaceTooSmall ,
{221} DE_ClearTable ,
{222} DE_MethodMasterTable,
{223} DE_NoTableNameLoc ,
{224} DE_AlterTransActive ,
{225} DE_DatasetMismatch ,
{226} DE_DropIndexIdentifier ,
{227} DE_NotAnIndexedField ,
{228} DE_MissingMeta ,
{229} DE_IndexingConnected ,
{230} DE_NotDesignTime ,
{231} DE_TableOperation ,
{232} DE_MissingSourceTable ,
{233} DE_IllegalCommandMode ,
{234} DE_RecordNotFound ,
{235} DE_NoUpdateWhere ,
{236} DE_NoBackup ,
{237} DE_InvalidIndexSpace ,
{238} DE_ViewerNoHandler,
{239} DE_CmdOptionNotFound ,
{240} DE_UndefinedMasterTable ,
{241} DE_DataDupTable,
{242} DE_DVNoSourceTable ,
{243} DE_FieldNameRequired ,
{244} DE_CtrlAllowed , { What kind of controls allowed }
{245} DE_DupController ,
{246} DE_CommandsMinimum ,
{247} DE_ControlMix ,
{248} DE_InvalidCommandName ,
{249} DE_DBNotIdle ,
{250} Reserved  ,
{251} DE_TableNotInDB ,
{252} Reserved ,
{253} DE_MissingMasterSource ,
{254} Reserved ,
{255} Reserved ,
{256} DE_ButtonAvailable ,
{257} Reserved ,
{258} Reserved ,
{259} DE_CtrlDataTypeMismatch ,
{260} DE_TableOutOfPath ,
{261} DE_LookupNotRegistered ,
{262} DE_LookupDelete ,
{263} DE_ReservedName ,
{264} DE_TreeViewSysField ,
{265} Reserved ,
{266} DE_MissingRootTable ,
{267} DE_FreeTables ,
{268} DE_RootDetail ,
{269} DE_InvalidChar ,
{270} DE_NotAParentTable ,
{271} DE_MissingChild ,
{272} DE_MissingParent ,
{273} DE_ParentEqChild ,
{274} DE_LibNotCreated ,
{275} DE_CircularTrigger ,
{276} DE_SysFieldLookup  ,
{277} DE_InvalidLookupRef ,
{278} DE_TreeViewChildTables ,
{279} DE_ManagerNotFound ,
{280} DE_SearchFieldOrder ,
{281} DE_TreeViewLabel ,
{282} DE_UnKnownTableClass ,
{283} DE_DupButtonType , {Field controller button props }
{284} DE_MasterTblNotRoot , { Field controller SetMasterTable }
{285} DE_CmdNotEnlisted , {Field controller, trying to set command mode when mode not enlisted }
{286} DE_CreateConnected ,
{287} Reserved ,
{288} DE_TableNoFields ,
{289} Reserved ,
{290} Reserved ,
{291} DE_MissingDBName ,
{292} DE_DBInvalid  ,
{293} Reserved ,
{294} Reserved ,
{295} DE_MissingInsertFields ,
{296} DE_InvalidMetaFile ,
{297} Reserved ,
{298} DE_InvalidDir ,
{299} DE_PanelCtrls ,
{300} Reserved ,
{301} DE_UpdateZoomed ,
{302} DE_ParentResolve ,
{303} Reserved ,
{304} DE_SiblingsIncomplete ,
{305} DE_ImageSourceRecord , { When adding immages to a image table
       the source record cannot come from antoher lookup table}
{306} Reserved ,
{307} DE_DBConstruct , { When importing schema }
{308} DE_OptDefaultNeeded ,
{309} DE_OptSearchAndNeeded ,
{310} DE_LooTableNotDefined1 ,
{311} DE_LooTableNotDefined2 ,
{312} DE_MissingSourceTable ,
{313} DE_TableNotAssigned ,
{314} DE_DacNotRegistered ,
{315} DE_LooFieldType ,
{316} DE_AutoIncTableEmpty ,
{317} DE_ValuePickList ,
{318} DE_LooUnique ,
{319} DE_LooTableNotDefined ,
{320} DE_LooFieldRef ,
{321} DE_LooIdent ,
{322} DE_ViewerTable ,
{323} DE_NotUserCalcField ,
{324} DE_ConnectByLiteral ,
{325} DE_ConnectByRoot ,
{326} Reserved ,
{327} Reserved ,
{328} DE_LogOpBtnType ,
{329} DE_LookupSelf ,
{330} DE_LooKeyDataTable ,
{331} DE_LooKeyPath ,
{332} DE_LooKeyNameOrNumber ,
{333} DE_LooSearchField ,
{334} DE_PropLock ,
{335} DE_FreeSpace ,
{336} DE_InvalidTableKey ,
{337} DE_MissingVALUES ,
{338} DE_MissingLookupTable ,
{339} DE_TBLErrorState ,
{340} DE_RecordSizeTooSmall,
{341} DE_DupFieldName ,
{342} DE_InvalidSortField ,
{343} DE_IllegalSortfield ,
{344} DE_ImgTableNotAssigned ,
{345} DE_TableNotEmpty ,
{346} DE_DBLoaded ,
{347} DE_DBNotLoaded ,
{348} DE_MissingMetaField ,
{349} DE_TableDLL,
{350} DE_ResSingleLine ,
{351} DE_MissingResElement ,
{352} DE_ResSearchControl ,
{353} DE_MissingResName ,
{354} DE_MissingResAddress ,
{355} DE_DupDBName ,
{356} DE_ImageSpecifier,
{357} '',
{358} ''
);


TRANSL_USER_ERRORS = 1;
TRANSL_TERMS = 2;

implementation

end.
