{**************************************************************************** }
{ Version 1.0 2021 }
{ Author: Oivind Muller }
{ }
{ This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <https://www.gnu.org/licenses/>.
{ }
{ **************************************************************************** }
unit PropertyProvider;

interface

uses
  Windows, SysUtils, ActiveX, Classes, VCL.Graphics, {ShellCtrls,} SHLObj, Propsys, Variants;

const
  { Property key definitions }

//https://docs.microsoft.com/nb-no/windows/desktop/properties/props-system-photo-exposureprogramtext
//https://docs.microsoft.com/nb-no/windows/desktop/api/propidl/ns-propidl-tagpropvariant

 PropKey_Med_Duration : TpropertyKey =
   ( fmtid : '{64440490-4C8B-11D1-8B70-080036B11A03}';
     pid : 3);
 PropKey_Med_DateEncoded : TpropertyKey =
   ( fmtid : '{2E4B640D-5019-46D8-8881-55414CC5CAA0}';
     pid : 100);
 PropKey_Photo_Aperture : TpropertyKey =
   ( fmtid : '{14B81DA1-0135-4D31-96D9-6CBFC9671A99}';
     pid : 37378);
 PropKey_Photo_Make : TpropertyKey =
   ( fmtid : '{14B81DA1-0135-4D31-96D9-6CBFC9671A99}';
     pid : 271);
 PropKey_Photo_Model : TpropertyKey =
   ( fmtid : '{14B81DA1-0135-4D31-96D9-6CBFC9671A99}';
     pid : 272);
 PropKey_Photo_LensMake : TpropertyKey =
   ( fmtid : '{E6DDCAF7-29C5-4F0A-9A68-D19412EC7090}';
     pid : 100);
 PropKey_Photo_LensModel : TpropertyKey =
   ( fmtid : '{E1277516-2B5F-4869-89B1-2E585BD38B7A}';
     pid : 100);
 PropKey_Photo_CamDate : TpropertyKey =
   ( fmtid : '{14B81DA1-0135-4D31-96D9-6CBFC9671A99}';
     pid : 36867);
 PropKey_Photo_ExposureProgram : TpropertyKey =
   ( fmtid : '{14B81DA1-0135-4D31-96D9-6CBFC9671A99}';
     pid : 34850);
 PropKey_Photo_ExposureProgramText : TpropertyKey =
   ( fmtid : '{FEC690B7-5F30-4646-AE47-4CAAFBA884A3}';
     pid : 100);
 PropKey_Photo_ExposureTime : TpropertyKey =
   ( fmtid : '{14B81DA1-0135-4D31-96D9-6CBFC9671A99}';
     pid : 33434);
 PropKey_Photo_ExposureBias : TpropertyKey =
   ( fmtid : '{14B81DA1-0135-4D31-96D9-6CBFC9671A99}';
     pid : 37380);
 PropKey_Photo_Flash : TpropertyKey =
   ( fmtid : '{14B81DA1-0135-4D31-96D9-6CBFC9671A99}';
     pid : 37385);
 PropKey_Photo_FNumber : TpropertyKey =
   ( fmtid : '{14B81DA1-0135-4D31-96D9-6CBFC9671A99}';
     pid : 33437);
 PropKey_Photo_FocalLength : TpropertyKey =
   ( fmtid : '{14B81DA1-0135-4D31-96D9-6CBFC9671A99}';
     pid : 37386);
 PropKey_Photo_ISO : TpropertyKey =
   ( fmtid : '{14B81DA1-0135-4D31-96D9-6CBFC9671A99}';
     pid : 34855);
 PropKey_Photo_MaxAperture : TpropertyKey =
   ( fmtid : '{08F6D7C2-E3F2-44FC-AF1E-5AA5C81A2D3E}';
     pid : 100);
 PropKey_Photo_Orientation : TpropertyKey =
   ( fmtid : '{14B81DA1-0135-4D31-96D9-6CBFC9671A99}';
     pid : 274);
 PropKey_Photo_ProgramMode : TpropertyKey =
   ( fmtid : '{6D217F6D-3F6A-4825-B470-5F03CA2FBE9B}';
     pid : 100);
 PropKey_Photo_ProgramModeText : TpropertyKey =
   ( fmtid : '{7FE3AA27-2648-42F3-89B0-454E5CB150C3}';
     pid : 100);
 PropKey_Photo_MeteringMode : TpropertyKey =
   ( fmtid : '{14B81DA1-0135-4D31-96D9-6CBFC9671A99}';
     pid : 37383);
 PropKey_Photo_MeteringModeText : TpropertyKey =
   ( fmtid : '{F628FD8C-7BA8-465A-A65B-C5AA79263A9E}';
     pid : 100);
 PropKey_Photo_Shutterspeed : TpropertyKey =
   ( fmtid : '{14B81DA1-0135-4D31-96D9-6CBFC9671A99}';
     pid : 37377);
 PropKey_Photo_XResolution : TpropertyKey =
   ( fmtid : '{CFC08D97-C6F7-4484-89DD-EBEF4356FE76}';
     pid : 100);
 PropKey_Photo_YResolution : TpropertyKey =
   ( fmtid : '{4FFFE4D0-914F-4AC4-8D6F-C9C61DE169B1}';
     pid : 100);
 PropKey_Photo_WhiteBalance : TpropertyKey =
   ( fmtid : '{EE3D3D8A-5381-4CFA-B13B-AAF66B5F4EC9}';
     pid : 100);

 PropKey_Audio_Bitrate : TpropertyKey =
   ( fmtid : '{E6822FEE-8C17-4D62-823C-8E9CFCBD1D5C}';
     pid : 100);
 PropKey_Audio_Format : TpropertyKey =
   ( fmtid : '{64440490-4C8B-11D1-8B70-080036B11A03}';
     pid : 2);

 PropKey_Video_Bitrate : TpropertyKey =
   ( fmtid : '{64440491-4C8B-11D1-8B70-080036B11A03}';
     pid : 8);
 PropKey_Video_FrameHeight : TpropertyKey =
   ( fmtid : '{64440491-4C8B-11D1-8B70-080036B11A03}';
     pid : 4);
 PropKey_Video_FrameWidth : TpropertyKey =
   ( fmtid : '{64440491-4C8B-11D1-8B70-080036B11A03}';
     pid : 3);
 PropKey_Video_FrameRate : TpropertyKey =
   ( fmtid : '{64440491-4C8B-11D1-8B70-080036B11A03}';
     pid : 6);

 PropKey_Image_XResolution : TpropertyKey =
   ( fmtid : '{6444048F-4C8B-11D1-8B70-080036B11A03}';
     pid : 5);
 PropKey_Image_YResolution : TpropertyKey =
   ( fmtid : '{6444048F-4C8B-11D1-8B70-080036B11A03}';
     pid : 6);
 PropKey_Image_XSize : TpropertyKey =
   ( fmtid : '{6444048F-4C8B-11D1-8B70-080036B11A03}';
     pid : 3);
 PropKey_Image_YSize : TpropertyKey =
   ( fmtid : '{6444048F-4C8B-11D1-8B70-080036B11A03}';
     pid : 4);
 PropKey_Image_BitDepth : TpropertyKey =
   ( fmtid : '{6444048F-4C8B-11D1-8B70-080036B11A03}';
     pid : 7);

 PropKey_Gps_Altitude : TpropertyKey =
   ( fmtid : '{827EDB4F-5B73-44A7-891D-FDFFABEA35CA}';
     pid : 100);
 PropKey_Gps_Latitude : TpropertyKey =
   ( fmtid : '{8727CFFF-4868-4EC6-AD5B-81B98521D1AB}';
     pid : 100);
 PropKey_Gps_Longitude : TpropertyKey =
   ( fmtid : '{C4C4DBB2-B593-466B-BBDA-D03D27D5E43A}';
     pid : 100);

 { Prop ids: }

 PID_Med_Duration = 100;
 PID_Med_DateEncoded = 143;
 PID_Photo_Aperture = 102;
 PID_Photo_Make = 103;
 PID_Photo_Model = 104;
 PID_Photo_LensMake = 140;
 PID_Photo_LensModel = 141;
 PID_Photo_CamDate = 105;
 PID_Photo_ExposureProgram = 106;
 PID_Photo_ExposureProgramText = 107;
 PID_Photo_ExposureTime = 108;
 PID_Photo_ExposureBias = 139;
 PID_Photo_Flash  = 109;
 PID_Photo_FNumber = 110;
 PID_Photo_FocalLength = 111;
 PID_Photo_ISO  = 112;
 PID_Photo_MaxAperture  = 113;
 PID_Photo_Orientation  =  114;
 PID_Photo_ProgramMode = 115;
 PID_Photo_ProgramModeText  = 116;
 PID_Photo_MeteringMode = 117;
 PID_Photo_MeteringModeText = 118;
 PID_Photo_Shutterspeed = 119;
 PID_Photo_XResolution = 120;
 PID_Photo_YResolution = 121;
 PID_Photo_WhiteBalance = 142;
 PID_Audio_Bitrate = 122;
 PID_Audio_Format = 123;
 PID_Video_Bitrate = 124;
 PID_Video_FrameHeight = 125;
 PID_Video_FrameWidth = 126;
 PID_Video_FrameRate = 127;
 PID_Image_XResolution  = 128;
 PID_Image_YResolution  = 129;
 PID_Image_XSize = 130;
 PID_Image_YSize = 131;
 PID_Image_BitDepth = 132;
 PID_Gps_Altitude = 133;
 PID_Gps_Latitude = 134;
 PID_Gps_Longitude = 135;
 PID_File_Size = 136;
 PID_File_Date = 137;
 PID_File_FriendlySize = 138;


 PropsetVideo = 1;
 PropsetPhoto = 2;
 PropsetAudio = 3;

 DT_STRING = 1;
 DT_INTEGER = 2;
 DT_FLOAT = 3;
 DT_BOOLEAN = 4;
 DT_DATE = 5;

type
   TPropValue = record
     PropID : integer;
     Value : Variant;
   end;

   TPropSet = array of TPropValue;

   TPropertyProvider = class
     private
       FPropertyStore : iPropertyStore;
       FFileName : string;
     public
       constructor Create(const AFileName : string);
       destructor Destroy; override;
       function GetPropValueAsVariant(PropID: integer) : Variant;
       function GetPropValueAsString(PropID: integer) : string;
       function SetPropValue(PropId : integer; Value : Variant) : HResult;
   end;


function FileTimeToDateTime(const FileTime: TFileTime): TDateTime;
function PidOf(const PidName : string) : integer;
function PidNameOf(const Pid : integer) : string;
function DataTypeOfPID(const PID : integer) : integer;

implementation uses math;

function PidOf(const PidName : string) : integer;
begin
    Result := -1;
    if SameText(PidName, 'PID_Med_Duration') then Result := PID_Med_Duration
    else if SameText(PidName, 'PID_Med_DateEncoded') then Result := PID_Med_DateEncoded
    else if SameText('PID_Photo_Aperture', PidName) then Result := PID_Photo_Aperture
    else if SameText('PID_Photo_Make',PidName) then Result := PID_Photo_Make
    else if SameText('PID_Photo_Model', PidName) then Result := PID_Photo_Model
    else if SameText('PID_Photo_LensMake', PidName) then Result := PID_Photo_LensMake
    else if SameText('PID_Photo_LensModel', PidName) then Result := PID_Photo_LensModel
    else if SameText('PID_Photo_CamDate', PidName) then Result := PID_Photo_CamDate
    else if SameText('PID_Photo_ExposureProgram', Pidname) then Result := PID_Photo_ExposureProgram
    else if SameText('PID_Photo_ExposureProgramText', Pidname) then Result := PID_Photo_ExposureProgramText
    else if SameText('PID_Photo_ExposureTime', Pidname) then Result := PID_Photo_ExposureTime
    else if SameText('PID_Photo_ExposureBias', Pidname) then Result := PID_Photo_ExposureBias
    else if SameText('PID_Photo_Flash', Pidname) then Result := PID_Photo_Flash
    else if SameText('PID_Photo_FNumber', Pidname) then Result := PID_Photo_FNumber
    else if SameText('PID_Photo_FocalLength', Pidname) then Result := PID_Photo_FocalLength
    else if SameText('PID_Photo_ISO', Pidname) then Result := PID_Photo_ISO
    else if SameText('PID_Photo_MaxAperture', Pidname) then Result := PID_Photo_MaxAperture
    else if SameText('PID_Photo_Orientation', Pidname) then Result := PID_Photo_Orientation
    else if SameText('PID_Photo_ProgramMode', Pidname) then Result := PID_Photo_ProgramMode
    else if SameText('PID_Photo_ProgramModeText', Pidname) then Result := PID_Photo_ProgramModeText
    else if SameText('PID_Photo_MeteringMode', Pidname) then Result := PID_Photo_MeteringMode
    else if SameText('PID_Photo_MeteringModeText', Pidname) then Result := PID_Photo_MeteringModeText
    else if SameText('PID_Photo_Shutterspeed', Pidname) then Result := PID_Photo_Shutterspeed
    else if SameText('PID_Photo_XResolution', Pidname) then Result := PID_Photo_XResolution
    else if SameText('PID_Photo_YResolution', Pidname) then Result := PID_Photo_YResolution
    else if SameText('PID_Photo_WhiteBalance', Pidname) then Result := PID_Photo_WhiteBalance
    else if SameText('PID_Audio_Bitrate', Pidname) then Result := PID_Audio_Bitrate
    else if SameText('PID_Audio_Format', Pidname) then Result := PID_Audio_Format
    else if SameText('PID_Video_Bitrate', Pidname) then Result := PID_Video_Bitrate
    else if SameText('PID_Video_FrameHeight', Pidname) then Result := PID_Video_FrameHeight
    else if SameText('PID_Video_FrameWidth', Pidname) then Result := PID_Video_FrameWidth
    else if SameText('PID_Video_FrameRate ', Pidname) then Result := PID_Video_FrameRate
    else if SameText('PID_Image_XResolution', Pidname) then Result := PID_Image_XResolution
    else if SameText('PID_Image_YResolution', Pidname) then Result := PID_Image_YResolution
    else if SameText('PID_Image_XSize', Pidname) then Result := PID_Image_XSize
    else if SameText('PID_Image_YSize', Pidname) then Result := PID_Image_YSize
    else if SameText('PID_Image_BitDepth', Pidname) then Result := PID_Image_BitDepth
    else if SameText('PID_Gps_Altitude', Pidname) then Result := PID_Gps_Altitude
    else if SameText('PID_Gps_Latitude', Pidname) then Result := PID_Gps_Latitude
    else if SameText('PID_Gps_Longitude', Pidname) then Result := PID_Gps_Longitude
    else if SameText('PID_File_Size', Pidname) then Result := PID_File_Size
    else if SameText('PID_File_Date', Pidname) then Result := PID_File_Date
    else if SameText('PID_File_FriendlySize', Pidname) then Result := PID_File_FriendlySize;
end;

function PidNameOf(const Pid : integer) : string;
begin
   Result := '';
   case PID of
     PID_Med_Duration : Result := 'PID_Med_Duration';
     PID_Med_DateEncoded : Result := 'PID_Med_DateEncoded';
     PID_Photo_ISO : Result := 'PID_Photo_ISO';
     PID_Photo_Orientation : Result := 'PID_Photo_Orientation';
     PID_Photo_ProgramMode : Result := 'PID_Photo_ProgramMode';
     PID_Photo_MeteringMode : Result := 'PID_Photo_MeteringMode';
     PID_Photo_XResolution : Result := 'PID_Photo_XResolution';
     PID_Photo_YResolution : Result := 'PID_Photo_YResolution';
     PID_Photo_WhiteBalance : Result := 'PID_Photo_WhiteBalance';
     PID_Audio_Bitrate : Result := 'PID_Audio_Bitrate';
     PID_Video_Bitrate : Result := 'PID_Video_Bitrate';
     PID_Video_FrameRate : Result := 'PID_Video_FrameRate';
     PID_Video_FrameWidth : Result := 'PID_Video_FrameWidth';
     PID_Video_FrameHeight : Result := 'PID_Video_FrameHeight';
     PID_Image_XResolution : Result := 'PID_Image_XResolution';
     PID_Image_YResolution : Result := 'PID_Image_YResolution';
     PID_Image_XSize : Result := 'PID_Image_XSize';
     PID_Image_YSize : Result := 'PID_Image_YSize';
     PID_Image_BitDepth : Result := 'PID_Image_BitDepth';
     PID_Photo_ExposureProgram : Result := 'PID_Photo_ExposureProgram';
     PID_Photo_ExposureBias : Result := 'PID_Photo_ExposureBias';
     PID_Photo_Make : Result := 'PID_Photo_Make';
     PID_Photo_Model : Result := 'PID_Photo_Model';
     PID_Photo_LensMake : Result := 'PID_Photo_LensMake';
     PID_Photo_LensModel : Result := 'PID_Photo_LensModel';
     PID_Photo_ProgramModeText : Result := 'PID_Photo_ProgramModeText';
     PID_Photo_MeteringModeText : Result := 'PID_Photo_MeteringModeText';
     PID_Photo_ExposureProgramText : Result := 'PID_Photo_ExposureProgramText';
     PID_Audio_Format : Result := 'PID_Audio_Format';
     PID_Photo_CamDate : Result := 'PID_Photo_CamDate';
     PID_Photo_Flash : Result := 'PID_Photo_Flash';
     PID_Photo_Aperture : Result := 'PID_Photo_Aperture';
     PID_Photo_ExposureTime : Result := 'PID_Photo_ExposureTime';
     PID_Photo_FNumber : Result := 'PID_Photo_FNumber';
     PID_Photo_FocalLength : Result := 'PID_Photo_FocalLength';
     PID_Photo_MaxAperture : Result := 'PID_Photo_MaxAperture';
     PID_Photo_Shutterspeed : Result := 'PID_Photo_Shutterspeed';
     PID_Gps_Altitude : Result := 'PID_Gps_Altitude';
     PID_Gps_Latitude : Result := 'PID_Gps_Latitude';
     PID_Gps_Longitude : Result := 'PID_Gps_Longitude';
     PID_File_Size : Result := 'PID_File_Size';
     PID_File_Date: Result := 'PID_File_Date';
     PID_File_FriendlySize : Result := 'PID_File_FriendlySize';
    end;
end;

function DataTypeOfPID(const PID : integer) : integer;
begin
    Result := 0;
    case PID of
     PID_Med_Duration, PID_Photo_ISO,  PID_Photo_Orientation, PID_Photo_ProgramMode,  PID_Photo_MeteringMode,
     PID_Photo_XResolution, PID_Photo_YResolution, PID_Photo_WhiteBalance, PID_Audio_Bitrate,
     PID_Video_Bitrate, PID_Video_FrameRate, PID_Video_FrameWidth, PID_Video_FrameHeight,
     PID_Image_XResolution, PID_Image_YResolution, PID_Image_XSize, PID_Image_YSize,
     PID_Image_BitDepth, PID_Photo_ExposureProgram, PID_File_Size : Result := DT_INTEGER;

     PID_Photo_Make, PID_Photo_Model, PID_Photo_LensMake, PID_Photo_LensModel, PID_Photo_ProgramModeText,
     PID_Photo_MeteringModeText, PID_Photo_ExposureProgramText, PID_Audio_Format, PID_File_FriendlySize : Result := DT_STRING;

     PID_Photo_CamDate, PID_File_Date, PID_Med_DateEncoded : Result := DT_DATE;

     PID_Photo_Flash : Result := DT_BOOLEAN;

     PID_Photo_Aperture, PID_Photo_ExposureTime, PID_Photo_ExposureBias, PID_Photo_FNumber,  PID_Photo_FocalLength,
     PID_Photo_MaxAperture, PID_Photo_Shutterspeed, PID_Gps_Altitude, PID_Gps_Latitude,
     PID_Gps_Longitude : Result := DT_FLOAT;

    end;
end;

function VTOfPID(const PID : integer) : word;
begin
     Result := 0;
     case PID of
       {vt_R8}
        PID_Photo_Aperture,
        PID_Photo_ExposureTime,
        PID_Photo_ExposureBias,
        PID_Photo_FNumber,
        PID_Photo_FocalLength,
        PID_Photo_MaxAperture,
        PID_Photo_XResolution,
        PID_Photo_YResolution,
        PID_Image_XResolution,
        PID_Image_YResolution,
        PID_Gps_Altitude,
        PID_Photo_Shutterspeed : begin Result:= vt_R8; end;

       {vt_lpwstr}
       PID_Photo_Make,
       PID_Photo_Model,
       PID_Photo_LensMake,
       PID_Photo_LensModel,
       PID_Photo_ExposureProgramText,
       PID_Photo_ProgramModeText,
       PID_Photo_MeteringModeText,
       PID_Audio_Format: begin Result:= vt_lpwstr; end;

       {vt_filetime}
       PID_Photo_CamDate: Result:= vt_filetime;
       PID_Med_DateEncoded : Result := vt_FileTime;

       {vt_ui1 one byte unsigned}
       PID_Photo_Flash : begin Result:= vt_ui1; end;

       {vt_ui2 to byte unsigned}
       PID_Photo_ISO,
       PID_Photo_Orientation :Result:= vt_ui2;

       {vt ui4, 4 byte unsigned int}
        PID_Photo_WhiteBalance,
        PID_Photo_ExposureProgram,
        PID_Photo_ProgramMode,
        PID_Photo_MeteringMode,
        PID_Video_Bitrate,
        PID_Audio_Bitrate,
        PID_Video_FrameHeight,
        PID_Video_FrameWidth,
        PID_Video_FrameRate,
        PID_Image_XSize,
        PID_Image_YSize,
        PID_Image_BitDepth : Result:= vt_ui4;

        {vt_ui8 8 byte unsigned int}
        PID_Med_Duration : Result:= vt_ui8;
        { Double array}
        PID_Gps_Latitude,
        PID_Gps_Longitude :  Result:= vt_vector or vt_r8
     end;
end;

function FileTimeToDateTime(const FileTime: TFileTime): TDateTime;
var
    ModifiedTime: TFileTime;
    SystemTime: TSystemTime;
  begin
    Result := 0;
   if (FileTime.dwLowDateTime = 0) and (FileTime.dwHighDateTime = 0) then
    Exit;
   try
    FileTimeToLocalFileTime(FileTime, ModifiedTime);
    FileTimeToSystemTime(ModifiedTime, SystemTime);
    Result := SystemTimeToDateTime(SystemTime);
   except
    Result := Now;  // Something to return in case of error
   end;
end;

function DateTimeToFileTime(const DT: TDateTime): TFileTime;
var
    SystemTime: TSystemTime;
  begin
    try
    Result.dwLowDateTime := 0;
    Result.dwHighDateTime := 0;
    DateTimeToSystemTime(DT, SystemTime);
    SystemTimeToFiletime(SystemTime, Result);
   except
      // Something to return in case of error
   end;
end;

procedure GetPropKey(const PropID : integer; var PropKey : TPropertyKey);
begin
   case PropId of
    PID_Med_Duration: PropKey := PropKey_Med_Duration;
    PID_Med_DateEncoded : PropKey := PropKey_Med_DateEncoded;
    PID_Photo_Aperture: PropKey := PropKey_Photo_Aperture;
    PID_Photo_Make : PropKey := PropKey_Photo_Make;
    PID_Photo_Model: PropKey := PropKey_Photo_Model;
    PID_Photo_LensMake : PropKey := PropKey_Photo_LensMake;
    PID_Photo_LensModel : PropKey := PropKey_Photo_LensModel;
    PID_Photo_CamDate: PropKey := PropKey_Photo_CamDate;
    PID_Photo_ExposureProgram : PropKey := PropKey_Photo_ExposureProgram;
    PID_Photo_ExposureProgramText: PropKey := PropKey_Photo_ExposureProgramText;
    PID_Photo_ExposureTime: PropKey := PropKey_Photo_ExposureTime;
    PID_Photo_ExposureBias: PropKey := PropKey_Photo_ExposureBias;
    PID_Photo_Flash: PropKey := PropKey_Photo_Flash;
    PID_Photo_FNumber: PropKey := PropKey_Photo_FNumber;
    PID_Photo_FocalLength: PropKey := PropKey_Photo_FocalLength;
    PID_Photo_ISO: PropKey := PropKey_Photo_ISO;
    PID_Photo_MaxAperture: PropKey := PropKey_Photo_MaxAperture;
    PID_Photo_Orientation: PropKey := PropKey_Photo_Orientation;
    PID_Photo_ProgramMode: PropKey := PropKey_Photo_ProgramMode;
    PID_Photo_ProgramModeText: PropKey := PropKey_Photo_ProgramModeText;
    PID_Photo_MeteringMode: PropKey := PropKey_Photo_MeteringMode;
    PID_Photo_MeteringModeText: PropKey := PropKey_Photo_MeteringModeText;
    PID_Photo_Shutterspeed: PropKey := PropKey_Photo_Shutterspeed;
    PID_Photo_XResolution: PropKey := PropKey_Photo_XResolution;
    PID_Photo_YResolution: PropKey := PropKey_Photo_YResolution;
    PID_Photo_WhiteBalance: PropKey := PropKey_Photo_WhiteBalance;
    PID_Audio_Bitrate: PropKey := PropKey_Audio_Bitrate;
    PID_Audio_Format: PropKey := PropKey_Audio_Format;
    PID_Video_Bitrate: PropKey := PropKey_Video_Bitrate;
    PID_Video_FrameHeight: PropKey := PropKey_Video_FrameHeight;
    PID_Video_FrameWidth: PropKey := PropKey_Video_FrameWidth;
    PID_Video_FrameRate : PropKey := PropKey_Video_FrameRate;
    PID_Image_XResolution: PropKey := PropKey_Image_XResolution;
    PID_Image_YResolution: PropKey := PropKey_Image_YResolution;
    PID_Image_XSize: PropKey := PropKey_Image_XSize;
    PID_Image_YSize: PropKey := PropKey_Image_YSize;
    PID_Image_BitDepth: PropKey := PropKey_Image_BitDepth;
    PID_Gps_Altitude : Propkey := Propkey_Gps_Altitude;
    PID_Gps_Latitude : Propkey := Propkey_Gps_Latitude;
    PID_Gps_Longitude : Propkey := Propkey_Gps_Longitude;
   end;
end;

constructor TPropertyProvider.Create(const AFileName: string);
var
   HR : HResult;
begin
 inherited Create;
 Hr := 0;
 if not Succeeded(hr) then
   raise Exception.Create('Com initialzation error');
 FFilename := AFilename;
 hr := SHGetPropertyStoreFromParsingName(PChar(AFilename), nil, GPS_READWRITE, IPropertyStore, FPropertyStore);
 if hr <> S_OK then
    raise Exception.Create('Error opening property store');
end;

destructor TPropertyProvider.Destroy;
begin
   inherited destroy;
end;

function TPropertyProvider.GetPropValueAsVariant(PropID : integer) : Variant;
var
  PropVar : TPropvariant;
  U : Int64;
  DT : TDateTime;
  PropKey : TPropertyKey;
  Cnt : integer;
  i : integer;
  P : pDouble;
  D : Double;
  hr : HRESULT;

  function ConvertBytes(const Bytes: Int64): string;
  const
    Description: Array [0 .. 8] of string = ('Bytes', 'KB', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB', 'YB');
  var
    i: Integer;
  begin
    i := 0;
    while Bytes > Power(1024, i + 1) do
      Inc(i);
    Result := FormatFloat('###0.##', Bytes / IntPower(1024, i)) + ' ' + Description[i];
  end;


  function GetFileAttr(const PropID : integer) : Variant;
  var
   info: TWin32FileAttributeData;
   U : int64;
  begin
    Result := Null;
    if GetFileAttributesEx(PWideChar(FFileName), GetFileExInfoStandard, @info) then
    begin
       if PropID = PID_FILE_SIZE then
         Result := Int64(info.nFileSizeLow) or Int64(info.nFileSizeHigh shl 32)
       else if PropID = PID_FILE_FRIENDLYSIZE then
       begin
         U := Int64(info.nFileSizeLow) or Int64(info.nFileSizeHigh shl 32);
         Result := ConvertBytes(U);
       end
       else if PropId = PID_FILE_DATE then
         Result := FileTimeToDateTime(info.ftCreationTime);
    end;
  end;

begin
   Result := Null;
   if PropID in [PID_File_Size..PID_File_FriendlySize] then
   begin
      Result := GetFileAttr(PropID);
      Exit;
   end;

   GetPropKey(PropID, PropKey);
   hr := FPropertyStore.GetValue(PropKey, PropVar);
   if not Succeeded(hr) then
     raise Exception.Create('Error getting property value');
   if PropVar.vt = vt_Empty then
   begin
    if PropID = PID_PHOTO_APERTURE then
    begin
       PropID := PID_PHOTO_FNUMBER;
       PropKey := PropKey_PHOTO_FNUMBER;
       FPropertyStore.GetValue(PropKey, PropVar);
       if PropVar.vt = vt_Empty then
       begin
         Result := Null;
         Exit;
       end;
     end
     else if PropID = PID_PHOTO_SHUTTERSPEED then
     begin
       PropID := PID_PHOTO_EXPOSURETIME;
       PropKey := PropKey_PHOTO_EXPOSURETIME;
       FPropertyStore.GetValue(PropKey, PropVar);
       if PropVar.vt = vt_Empty then
       begin
         Result := Null;
         Exit;
       end;
     end
     else
     begin
      Result := Null;
      Exit;
     end;
   end;
   case PropVar.vt of
      vt_i2: Result := PropVar.iVal;
      vt_i4: Result := PropVar.lVal;
      vt_i1: Result := PropVar.cVal;
      vt_ui1: Result := PropVar.bVal;
      vt_ui2: Result := PropVar.uiVal;
      vt_ui4: Result := PropVar.ulVal;
      vt_ui8 :
      begin
        U := PropVar.uhVal.QuadPart;
        Result := U;
      end;
      vt_int: Result := PropVar.iVal;
      vt_uint: Result := PropVar.uiVal;
      vt_i8:
      begin
        U := PropVar.uhVal.QuadPart;
        Result := U;
      end;
      vt_r4: Result := PropVar.fltVal;
      vt_r8: Result := PropVar.dblVal;
      vt_bool: Result := PropVar.boolVal;
      vt_date: Result := Propvar.date;
      vt_FileTime:
      begin
        DT := FileTimeToDateTime(Propvar.filetime);
        Result := DT;
      end;
      vt_lpstr : Result := string(PropVar.pszVal);
      vt_lpwstr : Result := string(PropVar.pwszVal);
      vt_vector or vt_r8 :
      begin
         Cnt := PropVar.cadbl.cElems;
         Result := VarArrayCreate([0, Cnt-1], varDouble);
         P := PropVar.cadbl.pElems;
         for i := 0 to Cnt-1 do
         begin
            D := P^;
            Result[i] := D;
            inc(P);
         end;
      end;
   end;
   U := 0;
   if PropId = PID_MED_Duration then
   begin
       U := Trunc(U/10000000);  { Convert to seconds }
       Result := U;
   end
   else if PropID = PID_PHOTO_FLASH then
   begin
      if (Result = 0)
      or (Result = 16)
      or (Result = 24)
      or (Result = 32) then
        Result := 0
      else
        Result := 1;
   end;

end;

function TPropertyProvider.GetPropValueAsString(PropID : integer) : string;
var
  V : Variant;
  N : Double;
begin
    Result := '';
    V := GetPropValueAsVariant(PropID);
    if VarIsNull(V) then
      Exit;
   if PropId = PID_PHOTO_EXPOSURETIME then
   begin
       N := V;
       if Int(N) = 0 then
       begin
         N := 1/N;
         Result := '1/' + FloatToStr(N);
       end
       else
         Result := VarToStr(V);
   end
   else if PropId = PID_PHOTO_FNUMBER then
   begin
      Result := 'F/'+VarToStr(V);
   end
   else
    Result := VarToStr(V);
end;

function TPropertyProvider.SetPropValue(PropID : integer; Value : Variant) : HResult;
var
  PKey : TPropertyKey;
  PropVar : TPropVariant;
  Dt : FileTime;
  U : int64;
  s : string;
begin
   PropVar.cadbl.pElems := nil;
   try
   if PropID in [PID_File_Size..PID_File_FriendlySize] then
   begin
        Result := STG_E_ACCESSDENIED;
        Exit;
   end;
   if not (PropId in
     [PID_PHOTO_ISO, PID_PHOTO_CAMDATE, PID_PHOTO_MAKE, PID_PHOTO_MODEL,
     PID_PHOTO_LENSMAKE, PID_PHOTO_LENSMODEL,
     PID_PHOTO_METERINGMODE, PID_PHOTO_METERINGMODETEXT, PID_PHOTO_FLASH])
   then
   begin
      Result := STG_E_ACCESSDENIED;
      Exit;
   end;

   GetPropKey(PropID, PKey);

   PropVar.vt := VtOfPid(PropID);
   case PropVar.vt of
      vt_i2: PropVar.iVal := Value;
      vt_i4: PropVar.lVal := Value;
      vt_i1: PropVar.cVal := Value;
      vt_ui1: PropVar.bVal := Value;
      vt_ui2: PropVar.uiVal := Value;
      vt_ui4: PropVar.ulVal := Value;
      vt_ui8 :
      begin
        U := Value;
        PropVar.uhVal.QuadPart := U;
      end;
      vt_int: PropVar.iVal := Value;
      vt_uint: PropVar.uiVal := Value;
      vt_i8:
      begin
        U := Value;
        PropVar.uhVal.QuadPart := U;
      end;
      vt_r4: PropVar.fltVal := Value;
      vt_r8: PropVar.dblVal := Value;
      vt_bool: PropVar.boolVal := Value;
      vt_date: Propvar.date := Value;
      vt_FileTime:
      begin
        DT := DateTimeToFileTime(Value);
        Propvar.filetime := DT;
      end;
      vt_lpstr :
      begin
        s := VarToStr(Value);
        PropVar.pszVal := PAnsiChar(AnsiString(s));
      end;
      vt_lpwstr :
      begin
        s := VarToStr(Value);
        PropVar.pwszVal := PWideChar(s);
      end;
      vt_vector or vt_r8 :
      begin
         {The only props using this type is longtitude and latidude.
         These are read only properties in the propety store.
         If any properties using this type are added, memory handling must be
         resolved. As it is now, avs occur}

         {Cnt := VarArrayHighBound(Value,1) + 1;
         PropVar.cadbl.cElems := Cnt;
         GetMem(PropVar.cadbl.pElems, Cnt);
         for I := 0 to VarArrayHighBound(Value, 1) do
         begin
             PropVar.cadbl.pElems^ := Value[i];
             inc(PropVar.cadbl.pElems);
         end;}
         sleep(0);
      end;
      else
      begin
        Result := e_Fail;
        Exit;
      end;
   end;
   Result := FPropertyStore.SetValue(PKey, PropVar);
   if Succeeded(Result) then
   begin
     Result := FPropertyStore.Commit;
   end ;

   finally
     //if PropVar.cadbl.pElems <> nil then
     //  FreeMem(PropVar.cadbl.pElems);
   end;
end;

end.

