{ Design time code}
unit Reg;

interface
uses classes, DesignIntf, DesignEditors, DesignMenus, SysUtils, ImageLibrary, ILActionBar;

type
 TILCtrlPanelEditor = class(TComponentEditor)
  function GetVerbCount : integer; override;
  function GetVerb(Index : integer) : string; override;
  procedure ExecuteVerb(Index : integer); override;
  procedure Edit; override;
 end;

 TILDataEditor = class(TComponentEditor)
  function GetVerbCount : integer; override;
  function GetVerb(Index : integer) : string; override;
  procedure ExecuteVerb(Index : integer); override;
  procedure Edit; override;
 end;

 TILTMEditor = class(TComponentEditor)
  function GetVerbCount : integer; override;
  function GetVerb(Index : integer) : string; override;
  procedure ExecuteVerb(Index : integer); override;
  procedure Edit; override;
  procedure PrepareItem(Index: Integer; const AItem: IMenuItem); override;
 end;

procedure Register;


implementation
uses VCL.Dialogs, ColnEdit, ILMessages;

procedure Register;
begin
  RegisterClass(TILDataTable);
  RegisterClass(TILLookupTable);
  RegisterClass(TILImageTable);
  RegisterClass(TILTreeViewTable);
  RegisterClass(TILFieldDefs);
  RegisterClass(TILFieldDef);
  RegisterClass(TILDataFields);
  RegisterClass(TILDataField);
  RegisterClass(TILPropertyList);
  RegisterClass(TILPropertyItem);
  RegisterClass(TILDatasetViewer);
  RegisterNoIcon([TILDataTable, TILLookupTable, TILImageTable, TILTreeViewTable]);
  RegisterComponents('ImageLibrary', [TTableManager, TILData, TILSQL, TILView, TILMemTable,
   TILDatasetViewer, TILTreeView, TILImage, TILActionBar, TILMessageBar, TILSearchGrid, TILUpdateGrid,
   TILControlPanel, TILLogOpButton, TILRecordViewer, TILResourceList]);
  RegisterComponentEditor(TILControlPanel, TILCtrlPanelEditor);
  RegisterComponentEditor(TILData, TILDataEditor);
  RegisterComponentEditor(TTableManager, TILTMEditor);
end;


{ TILCtrlPanelEditor }

function TILCtrlPanelEditor.GetVerbCount : integer;
begin
     Result := 2;
end;

function TILCtrlPanelEditor.GetVerb(Index : integer) : string;
var
  S : string;
begin
    Result := '?';
    if TILControlPanel(Component).DataSource <> nil then
    begin
      S := TILControlPanel(Component).DataSource.Name;
      if Index = 0 then
        Result := 'Add search controls from ' + S
      else if Index = 1 then
        Result := 'Add data controls from ' + S;
    end
    else
    begin
      S := '(DataSource must be set)';
      if Index = 0 then
        Result := 'Add search controls ' + S
      else if Index = 1 then
        Result := 'Add data controls ' + S;
    end
end;

procedure TILCtrlPanelEditor.ExecuteVerb(Index : integer);
begin
    inherited;
    if TILControlPanel(Component).DataSource = nil then
      ShowMessage('The DataSource property must point to the TILData component containing the fields to match the controls')
    else if Index = 0 then
     TILControlPanel(Component).GenerateControls(ctSearchControls)
    else if Index = 1 then
     TILControlPanel(Component).GenerateControls(ctDataControls)
end;

procedure TILCtrlPanelEditor.Edit;
begin
    inherited;
end;

{ TILDataEditor }

function TILDataEditor.GetVerbCount : integer;
begin
     Result := 1;
end;

function TILDataEditor.GetVerb(Index : integer) : string;
begin
    Result := '';
    case Index of
     0 : Result := 'DataFields editor ...'
    end;
end;

procedure TILDataEditor.ExecuteVerb(Index : integer);
begin
   inherited;
   case Index of
    0: ShowCollectionEditor(Designer, Component, TILData(Component).Datafields, 'DataFields');
   end;

end;

procedure TILDataEditor.Edit;
begin
    inherited;
end;

{ TILTMEditor TableManagher }

function TILTMEditor.GetVerbCount : integer;
begin
     Result := 4;
end;

function TILTMEditor.GetVerb(Index : integer) : string;
begin
    Result := '';
    case Index of
     0 : Result := 'QB: Create database ...';
     1 : Result := 'QB: Create child table ...';
     2 : Result := 'QB: Alter fields ...';
     3 :Result := 'Synchronize ...';
    end;
end;

procedure TILTMEditor.ExecuteVerb(Index : integer);
begin
   inherited;
   with Component as TTableManager do
   case Index of
    0: LaunchQuickBuilder(qbCreateDB);
    1: LaunchQuickBuilder(qbCreateChildTable);
    2: LaunchQuickBuilder(qbAlter);
    3: Synchronize;
   end;
end;

procedure TILTMEditor.PrepareItem(Index: Integer; const AItem: IMenuItem);
begin
   with Component as TTableManager do
   case Index of
    0: AItem.Enabled := (Database = '');
    1, 2: AItem.Enabled := (Database <> '');
    3: AItem.Enabled := FileExists(MetaName) and (MetaFileStamp <> GetFileLastModDate(MetaName));
   end;
end;

procedure TILTMEditor.Edit;
begin
    inherited;
end;



end.
