unit form_quickbuilder;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes,
  System.UiTypes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls, Vcl.Buttons,
  Vcl.ExtCtrls, System.Actions, Vcl.ActnList, ImageLibrary, Vcl.ComCtrls;

type
  Tfrm_QuickBuilder = class(TForm)
    Panel1: TPanel;
    Panel2: TPanel;
    Panel3: TPanel;
    ListBox1: TListBox;
    e_DBName: TEdit;
    Label1: TLabel;
    cb_Table: TComboBox;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    e_FieldName: TEdit;
    cb_Datatype: TComboBox;
    e_Decimals: TEdit;
    cbx_Indexed: TCheckBox;
    cbx_Unique: TCheckBox;
    cbx_Mandatory: TCheckBox;
    Label5: TLabel;
    Label6: TLabel;
    Button1: TButton;
    Button2: TButton;
    Panel4: TPanel;
    SpeedButton1: TSpeedButton;
    SpeedButton2: TSpeedButton;
    Label7: TLabel;
    ActionList1: TActionList;
    ac_Add: TAction;
    ac_Remove: TAction;
    ac_Apply: TAction;
    Label8: TLabel;
    e_Location: TEdit;
    Label9: TLabel;
    e_Caption: TEdit;
    cb_ParentTable: TComboBox;
    Label10: TLabel;
    cbx_Identifier: TCheckBox;
    Button3: TButton;
    Button4: TButton;
    ac_RebuildIndexes: TAction;
    ac_Optimize: TAction;
    cbx_MulVal: TCheckBox;
    StatusBar1: TStatusBar;
    SpeedButton3: TSpeedButton;
    procedure ListBox1Click(Sender: TObject);
    procedure ac_AddExecute(Sender: TObject);
    procedure ac_RemoveUpdate(Sender: TObject);
    procedure ac_RemoveExecute(Sender: TObject);
    procedure ac_ApplyExecute(Sender: TObject);
    procedure ac_ApplyUpdate(Sender: TObject);
    procedure e_FieldNameChange(Sender: TObject);
    procedure cb_DatatypeChange(Sender: TObject);
    procedure e_DecimalsChange(Sender: TObject);
    procedure cbx_IndexedClick(Sender: TObject);
    procedure cbx_UniqueClick(Sender: TObject);
    procedure cbx_MandatoryClick(Sender: TObject);
    procedure e_CaptionChange(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure cb_TableCloseUp(Sender: TObject);
    procedure cbx_IdentifierClick(Sender: TObject);
    procedure ac_RebuildIndexesExecute(Sender: TObject);
    procedure ac_RebuildIndexesUpdate(Sender: TObject);
    procedure ac_OptimizeUpdate(Sender: TObject);
    procedure ac_OptimizeExecute(Sender: TObject);
    procedure cbx_MulValClick(Sender: TObject);
    procedure ac_AddUpdate(Sender: TObject);
    procedure SpeedButton3Click(Sender: TObject);
  private
    { Private declarations }
    TblDef : TTableDef;
    OrigCount : integer;
    QB : TQuickBuilder;
    Table : TILTable;
    PropBuild : Boolean;
    ProgEvent : TProgressEvent;
    LastProgPst : integer;
    procedure SetProps;
    procedure SetEnables;
    procedure SetDefaultProps;
    function ConstraintChanged(const FieldName : string; const Prop : string; const NewVal : Boolean) : Boolean;
    procedure Progression(Sender: TObject; const Operation: TILOperation; Count, Progress: Integer);
  public
    { Public declarations }
    TM : TTableManager;
    Actn : TQuickMode;

    procedure Fill;
    procedure Clear;
    procedure InitChild;
    function FieldsChanged : TTableDef;
  end;

var
  frm_QuickBuilder: Tfrm_QuickBuilder;

implementation
uses ILMessages, WinAPI.SHLObj, VCL.FileCtrl;

{$R *.dfm}

procedure Tfrm_Quickbuilder.Fill;
var
  i : integer;
  Adjust : integer;
begin
   if cb_Table.Items.Count = 0 then
   begin
    cb_Table.Style := csDropDownList;
     for I := 0 to TMan.TableCount-1 do
      if not (TMan.Tables[i] is TILView) then
       cb_Table.Items.Add(TMan.Tables[i].TableName);
    cb_Table.ItemIndex := 0;
   end;
   ListBox1.Items.Clear;
   Table := TMan.TableOf(cb_Table.Text);
   Adjust := 1;
   for I := 0 to Table.Fields.Count-1 do
   begin
     ListBox1.Items.Add(Table.Fields.Items[i].Fieldname);
     TblDef.Fields[i+Adjust].FieldID := Table.Fields.Items[i].FieldID;
     TblDef.Fields[i+Adjust].IsDeleted := false;
     TblDef.Fields[i+Adjust].FieldName := Table.Fields.Items[i].Fieldname;
     TblDef.Fields[i+Adjust].Caption := Table.Fields.Items[i].ReportCaption;
     TblDef.Fields[i+Adjust].DataType := Table.Fields.Items[i].DataType;
     TblDef.Fields[i+Adjust].Decimals := Table.Fields.Items[i].Decimals;
     TblDef.Fields[i+Adjust].Indexed := Table.Fields.Items[i].Indexed;
     TblDef.Fields[i+Adjust].Unique := Table.Fields.Items[i].Unique;
     TblDef.Fields[i+Adjust].Mandatory := Table.Fields.Items[i].Mandatory;
     TblDef.Fields[i+Adjust].Identifier := Table.Fields.Items[i].Identifier;
   end;
   TblDef.FieldCount := Table.Fields.Count;
   OrigCount := TblDef.FieldCount;
   Listbox1.ItemIndex := 0;
   e_DBName.Text := TMan.Database;
   e_Location.Text := TMan.Location;
   e_Location.Enabled := false;
   e_DBName.Enabled := false;
   cb_ParentTable.Clear;
   for I := 0 to TMan.TableNodeCount-1 do
    cb_ParentTable.Items.Add(TMan.TreeTables[i].Tablename);
   if TILDataTable(Table).MasterSource <> nil then
     cb_ParentTable.ItemIndex := cb_ParentTable.Items.IndexOf(TILDataTable(Table).MasterSource.TableName);
   cb_ParentTable.Enabled := false;
   SetProps;
end;

procedure Tfrm_QuickBuilder.InitChild;
var
  i : integer;
begin
  Clear;
  for i := 0 to TMan.TableNodeCount-1 do
    cb_ParentTable.Items.Add(TMan.TreeTables[i].TableName);
  cb_Table.Text := '';
  cb_Table.Enabled := True;
  cb_ParentTable.ItemIndex := 0;
  e_DBName.Text := TMan.Database;
  e_Location.Text := TMan.Location;
  e_Location.Enabled := false;
  e_DBName.Enabled := false;
  cb_DataType.Enabled := True;
end;

procedure Tfrm_QuickBuilder.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
var
  FD : TTableDef;
  F : TILFieldDef;
  T : TILTable;
  i,j : integer;
  Ind : integer;
  SuccCount : integer;
  FName : string;
begin
  if ModalResult = mrCancel then
  begin
    CanClose := True;
    Exit;
  end;
  QB := TQuickbuilder.Create;
  QB.Mode := Actn;
  try
  if TMan.Database = '' then
  begin
     TblDef.DatabaseName := e_DBName.Text;
     TblDef.Location := e_Location.Text;
     TblDef.TableName := cb_Table.Text;
     try
       QB.CreateSimpleDB(TblDef);
     except
       Canclose := false;
       raise;
     end;
  end
  else if Actn = qbCreateChildTable then
  begin
    TblDef.TableName := cb_Table.Text;
    TblDef.ParentTable := cb_ParentTable.Text;
    try
       QB.CreateChildTable(TblDef);
     except
       Canclose := false;
       raise;
     end;
  end
  else
  begin
     FD := FieldsChanged;
     SuccCount := 0;
     T := TMan.TableOf(cb_Table.Text);
     for I := 0 to Listbox1.Count-1 do
     begin
       FName := Listbox1.Items[i];
       for j := 0 to ListBox1.Count-1 do
         begin
           if j = i then
             Continue;
           if SameText(Listbox1.Items[j], FName) then
             raise Exception.Create('Duplicate fieldname (' + FName + ')');
         end;
     end;
     try
     for I := 1 to FD.FieldCount do
     begin
       if FD.Fields[i].FieldID = -1 then
       { Field added }
       begin
         QB.AddField(cb_Table.Text, FD.Fields[i]);
         inc(SuccCount);
         Continue;
       end
       else if FD.Fields[i].IsDeleted then
       { Field deleted }
       begin
         Ind := T.Fields.IndexOfID(FD.Fields[i].FieldID);
         QB.DropField(cb_Table.Text, T.Fields.Items[Ind].FieldName);
         inc(SuccCount);
         Continue;
       end;

       Ind := T.Fields.IndexOfID(FD.Fields[i].FieldID);
       F := T.Fields.Items[Ind];
       if not SameText(FD.Fields[i].FieldName, F.FieldName) then
       begin
         QB.AlterField(cb_Table.Text, F.FieldName, ALTER_FIELDNAME, FD.Fields[i]);
         inc(SuccCount);
       end;
       if not SameText(FD.Fields[i].Caption, F.ReportCaption) then
       begin
         QB.AlterField(cb_Table.Text, F.FieldName, ALTER_CAPTION, FD.Fields[i]);
         inc(SuccCount);
       end;
       if FD.Fields[i].Datatype <> F.Datatype then
       begin
         QB.AlterField(cb_Table.Text, F.FieldName, ALTER_DATATYPE, FD.Fields[i]);
         inc(SuccCount);
       end;
       if FD.Fields[i].Decimals <> F.Decimals then
       begin
         QB.AlterField(cb_Table.Text, F.FieldName, ALTER_DECIMALS, FD.Fields[i]);
         inc(SuccCount);
       end;
       if FD.Fields[i].Indexed <> F.Indexed then
       begin
         QB.AlterField(cb_Table.Text, F.FieldName, ALTER_INDEXED, FD.Fields[i]);
         inc(SuccCount);
       end;
       if FD.Fields[i].Unique <> F.Unique then
       begin
         QB.AlterField(cb_Table.Text, F.FieldName, ALTER_UNIQUE, FD.Fields[i]);
         inc(SuccCount);
       end;
       if FD.Fields[i].Mandatory <> F.Mandatory then
       begin
         QB.AlterField(cb_Table.Text, F.FieldName, ALTER_MANDATORY, FD.Fields[i]);
         inc(SuccCount);
       end;
       if FD.Fields[i].Identifier <> F.Identifier then
       begin
         QB.AlterField(cb_Table.Text, F.FieldName, ALTER_IDENTIFIER, FD.Fields[i]);
         inc(SuccCount);
       end;
       if (FD.Fields[i].MultiVal and (F.MaxValues  = 1))
       or (not FD.Fields[i].MultiVal and (F.MaxValues  = 0))
       then
       begin
         QB.AlterField(cb_Table.Text, F.FieldName, ALTER_MULVAL, FD.Fields[i]);
         inc(SuccCount);
       end;
     end;

     ShowMessage(IntToStr(SuccCount) + ' aleration(s) effected');

     except on E:Exception do
     begin
       MessageDLG(E.Message + ' ' +
        IntTostr(SuccCount) + ' alterations(s) still effected.', mtError, [mbOk], 0);
       Clear;
       Fill;
       CanClose := false;
     end;
     end;

  end;
  finally
    QB.Free;
  end;
end;

procedure Tfrm_QuickBuilder.Progression(Sender: TObject; const Operation: TILOperation; Count, Progress: Integer);
var
  S : string;

begin
   if LastProgPst = Round(Progress / Count * 100)  then
     Exit;
   LastProgPst := Round(Progress / Count * 100);
   S := IntToStr(LastProgPst) + '%';
   if Operation = iloRemoveDeleted then
    S := 'Removing deleted records: ' + S
   else if Operation = iloRecreateImages then
    S := 'Removing unused images: ' + S
   else if Operation = iloExtractIndexes then
    S := 'Extracting indexes: ' + S
   else if Operation = iloBuildIndexes then
    S := 'Building indexes: ' + S
   else if Operation = iloFinished then
    S := '';
   Statusbar1.SimpleText := S;
end;

function Tfrm_QuickBuilder.ConstraintChanged(const FieldName : string; const Prop : string;
 const NewVal : Boolean) : Boolean;
var
  F : TILFieldDef;
  Indx : integer;
begin
   Result := false;
   if not NewVal then
     Exit;
   if Table = nil then
     Exit;
   if Table.IsTableEmpty then
     Exit;
   Indx := Table.Fields.IndexOf(FieldName);
   if Indx = -1 then
     Exit;
   F := Table.Fields.Items[Indx];
   if SameText(Prop, 'Indexed') then
   begin
   if not F.Indexed then
     Result := True;
   end
   else if SameText(Prop, 'Mandatory') then
   begin
   if not F.Mandatory then
     Result := True;
   end
   else if SameText(Prop, 'Unique') then
   begin
   if not F.Unique then
     Result := True;
   end
   else if SameText(Prop, 'Identifier') then
   begin
   if not F.Identifier then
     Result := True;
   end;
end;

procedure Tfrm_QuickBuilder.ListBox1Click(Sender: TObject);
begin
  SetProps;
end;

procedure Tfrm_QuickBuilder.ac_AddExecute(Sender: TObject);
begin
   ListBox1.Items.Add('NewField');
   ListBox1.ItemIndex := ListBox1.Count -1;
   SetDefaultProps;
   Inc(TBlDef.FieldCount);
   TblDef.Fields[TBlDef.FieldCount].IsDeleted := false;
   TblDef.Fields[TBlDef.FieldCount].FieldID := -1;
   TblDef.Fields[TBlDef.FieldCount].FieldName := e_FieldName.Text;
   TblDef.Fields[TBlDef.FieldCount].Caption := e_Caption.Text;
   TblDef.Fields[TBlDef.FieldCount].DataType := TDataType(cb_DataType.ItemIndex);
   TblDef.Fields[TBlDef.FieldCount].Decimals := StrToInt(e_decimals.Text);
   TblDef.Fields[TBlDef.FieldCount].Indexed := cbx_Indexed.Checked;
   TblDef.Fields[TBlDef.FieldCount].Unique := cbx_Unique.Checked;
   TblDef.Fields[TBlDef.FieldCount].Mandatory := cbx_Mandatory.Checked;
   e_FieldName.Text := 'NewField';
   e_FieldName.Enabled := True;

end;

procedure Tfrm_Quickbuilder.SetDefaultProps;
begin
  PropBuild := True;
  e_FieldName.Text := '';
  e_FieldName.Enabled := True;
  e_Caption.Text := '';
  e_Caption.Enabled := True;
  cb_DataType.ItemIndex := 0;
  cb_DataType.Enabled := True;
  e_Decimals.Text := '-1';
  e_Decimals.Enabled := True;
  cbx_Indexed.Checked := false;
  cbx_Indexed.Enabled := True;
  cbx_Mandatory.Checked := false;
  cbx_Mandatory.Enabled := True;
  cbx_Unique.Checked := false;
  cbx_Unique.Enabled := True;
  PropBuild := false;
end;

procedure Tfrm_QuickBuilder.ac_AddUpdate(Sender: TObject);
begin
  ac_Add.Enabled := True;//csDesigning in Tman.ComponentState;
end;

procedure Tfrm_QuickBuilder.ac_ApplyExecute(Sender: TObject);
begin
  ModalResult := mrOK;
end;

procedure Tfrm_QuickBuilder.ac_ApplyUpdate(Sender: TObject);
var
  Enabl : Boolean;
  TD : TTableDef;
begin
    Enabl := True;
    if TMan.Database = '' then
    begin
       if Trim(e_DBName.Text) = '' then
         Enabl := false
       else if Trim(e_Location.Text) = '' then
         Enabl := false
       else if Trim(cb_Table.Text) = '' then
         Enabl := false
       else if ListBox1.Count = 0 then
         Enabl := false;
    end
    else
    begin
      if Listbox1.Count= 0 then
        Enabl :=  false
      else
      begin
        TD := FieldsChanged;
        Enabl := TD.FieldCount > 0;
      end;
    end;
    ac_Apply.Enabled := Enabl;
    if Actn in [qbCreateDB, qbCreateChildTable] then
      cb_Table.Enabled := True
    else
      cb_Table.Enabled := not Enabl;
end;

procedure Tfrm_QuickBuilder.ac_OptimizeExecute(Sender: TObject);
var
  FD : TTableDef;
begin
   FD := FieldsChanged;
   if FD.FieldCount > 0 then
   begin
     ShowMessage('Please commit your changes before executing this function');
     Exit;
   end;
  if MessageDlg('Remove unused items from the database? This might take several minutes.',
   mtConfirmation, [mbYes, mbNo], 0) = mrYes then
  begin
    LastProgPst := 0;
    ProgEvent := TMan.OnProgress;
    TMan.OnProgress := Progression;
    Screen.Cursor := crHourGlass;

    try
     TMan.RemoveGarbage;
     ShowMessage('Optimizing successfully concluded!');
    finally
      Screen.Cursor := crDefault;
      TMan.OnProgress := ProgEvent;
    end;
  end;
end;

procedure Tfrm_QuickBuilder.ac_OptimizeUpdate(Sender: TObject);
var
  FD : TTableDef;
  Enabl : Boolean;
begin
   Enabl := (Actn = qbAlter) and TMan.Connected and (csDesigning in Tman.ComponentState);
   if Enabl then
   begin
    FD := FieldsChanged;
    Enabl := FD.FieldCount = 0;
   end;
   ac_Optimize.Enabled := Enabl;
end;

procedure Tfrm_QuickBuilder.ac_RebuildIndexesExecute(Sender: TObject);
begin
   if MessageDlg('Rebuild indexes for table ' + Table.TableName + '?',
   mtConfirmation, [mbYes, mbNo], 0) = mrYes then
   begin
    LastProgPst := 0;
    ProgEvent := TMan.OnProgress;
    TMan.OnProgress := Progression;
    Screen.Cursor := crHourGlass;
    try
     Table.DBAdmin.RebuildIndexes;
     ShowMessage('Indexes rebuilt!');
    finally
     Screen.Cursor := crHourGlass;
     TMan.OnProgress := ProgEvent;
    end;
   end;
end;

procedure Tfrm_QuickBuilder.ac_RebuildIndexesUpdate(Sender: TObject);
var
  Enabl : Boolean;
  i : integer;
  HasIndx : boolean;
  FD : TTableDef;
begin
   Enabl := (Actn = qbAlter) and TMan.Connected and not (csDesigning in Tman.ComponentState);
   if Enabl then
   begin
    FD := FieldsChanged;
     Enabl := (FD.FieldCount = 0);
    if Enabl then
    begin
     HasIndx := false;
     for I := 0 to Table.Fields.Count-1 do
      if Table.Fields.Items[i].Indexed then
      begin
        HasIndx := True;
        Break;
      end;
      Enabl := HasIndx;
    end;
    if Enabl then
      Enabl := not Table.IsTableEmpty;
   end;
   ac_RebuildIndexes.Enabled := Enabl;
end;

procedure Tfrm_QuickBuilder.ac_RemoveExecute(Sender: TObject);
var
  Ind : integer;
begin
   Ind := ListBox1.ItemIndex;
    TblDef.Fields[Ind +1].IsDeleted := True;
    ListBox1.Items.Delete(Ind);
    if ListBox1.Count > 0 then
    begin
      if Ind = ListBox1.Count then
      begin
        dec(Ind);
      end;
      ListBox1.ItemIndex := Ind;
      SetProps;
    end
    else
    begin
      SetDefaultProps;
    end;
end;

procedure Tfrm_QuickBuilder.ac_RemoveUpdate(Sender: TObject);
var
  Enabl : Boolean;
  S : string;
begin
  Enabl := (Listbox1.Count > 1) and (Listbox1.ItemIndex > -1)
  and (csDesigning in TMan.ComponentState);
  if Enabl then
  begin
    S := Listbox1.Items[ListBox1.ItemIndex];
    Enabl := not SameText(S, 'SYS_LinkID')
    and not SameText(S, 'SYS_SiblingIndex')
    and not SameText(S, 'SYS_Level');
  end;
  ac_Remove.Enabled := Enabl;
end;

procedure Tfrm_QuickBuilder.cbx_IdentifierClick(Sender: TObject);
begin
   if PropBuild then
     Exit;
   if Listbox1.ItemIndex > -1  then
      TblDef.Fields[ListBox1.ItemIndex+1].Identifier := cbx_Identifier.Checked;
   if ConstraintChanged(TblDef.Fields[ListBox1.ItemIndex+1].FieldName, 'Identifier', cbx_Identifier.Checked) then
     begin
       MessageDlg('Note! Adding an Identifer when the table contains data, ' +
       'may violate table integrity.', mtWarning, [mbOk], 0);
     end;
end;

procedure Tfrm_QuickBuilder.cbx_IndexedClick(Sender: TObject);
begin
     if PropBuild then
       Exit;
     if Listbox1.ItemIndex > -1  then
      TblDef.Fields[ListBox1.ItemIndex+1].Indexed := cbx_Indexed.Checked;
     if ConstraintChanged(TblDef.Fields[ListBox1.ItemIndex+1].FieldName, 'Indexed', cbx_Indexed.Checked) then
     begin
       MessageDlg('Note! Flagging a field as indexed, requires the table to be reindexed. ' +
       'Please commit your changes and then run Rebuild indexes by launching this dialog runtime.', mtWarning, [mbOk], 0);
     end;
end;

procedure Tfrm_QuickBuilder.cbx_MandatoryClick(Sender: TObject);
begin
   if PropBuild then
       Exit;
   if Listbox1.ItemIndex > -1  then
      TblDef.Fields[ListBox1.ItemIndex+1].Mandatory := cbx_Mandatory.Checked;
   if ConstraintChanged(TblDef.Fields[ListBox1.ItemIndex+1].FieldName, 'Mandatory', cbx_Mandatory.Checked) then
     begin
       MessageDlg('Note! Setting Mandatory when the table contains data, ' +
       'may violate table integrity.', mtWarning, [mbOk], 0);
     end;
end;

procedure Tfrm_QuickBuilder.cbx_MulValClick(Sender: TObject);
begin
    if PropBuild then
       Exit;
    if Listbox1.ItemIndex > -1  then
      TblDef.Fields[ListBox1.ItemIndex+1].MultiVal := cbx_MulVal.Checked;
end;

procedure Tfrm_QuickBuilder.cbx_UniqueClick(Sender: TObject);
begin
    if PropBuild then
       Exit;
    if Listbox1.ItemIndex > -1  then
      TblDef.Fields[ListBox1.ItemIndex+1].Unique := cbx_Unique.Checked;
    if ConstraintChanged(TblDef.Fields[ListBox1.ItemIndex+1].FieldName, 'Unique', cbx_Unique.Checked) then
     begin
       MessageDlg('Note! Setting Unique when the table contains data, ' +
       'may violate table integrity.', mtWarning, [mbOk], 0);
     end;
end;

procedure Tfrm_QuickBuilder.cb_DatatypeChange(Sender: TObject);
begin
  SetEnables;
  if PropBuild then
       Exit;
  if ListBox1.ItemIndex > -1 then
    TblDef.Fields[ListBox1.ItemIndex+1].DataType := TDataType(cb_DataType.ItemIndex);
end;

procedure Tfrm_QuickBuilder.cb_TableCloseUp(Sender: TObject);
begin
  if Tman.TableOf(cb_Table.Text) <> Table then
  if Actn = qbAlter then
    Fill;
end;

procedure Tfrm_Quickbuilder.Clear;
begin
  PropBuild := True;
  try
  e_DbName.Text := '';
  e_DBName.Enabled := True;
  cb_ParentTable.Clear;
  cb_Table.Style := csSimple;
  cb_Table.Text := '';
  cb_Table.Enabled := True;
  cb_DataType.Enabled := True;
  Listbox1.Clear;
  SetDefaultProps;
  TblDef.FieldCount := 0;
  finally
    PropBuild := false;
  end;
end;

procedure Tfrm_QuickBuilder.e_CaptionChange(Sender: TObject);
begin
    if ListBox1.ItemIndex > -1 then
       TblDef.Fields[ListBox1.ItemIndex+1].Caption := e_Caption.Text;
end;

procedure Tfrm_QuickBuilder.e_DecimalsChange(Sender: TObject);
var
  N : integer;
begin
  if ListBox1.ItemIndex = -1 then
  begin
   e_decimals.Text := '';
   exit;
  end;
  if e_decimals.Text <> '' then
  begin
    if e_decimals.Text = '-' then
      e_decimals.Text := '-1';
    if not TryStrToInt(e_decimals.Text, N) then
    begin
      e_decimals.Text := InttoStr(TblDef.Fields[ListBox1.ItemIndex+1].Decimals);
      raise Exception.Create('Illegal number');
    end;
    if (StrToInt(e_decimals.Text) < -1) or (StrToInt(e_decimals.Text) > 4) then
    begin
      e_decimals.Text := InttoStr(TblDef.Fields[ListBox1.ItemIndex+1].Decimals);
      raise Exception.Create('Must be a number between -1 and 4');
    end;
  end;

  if ListBox1.ItemIndex > -1 then
  begin
    if e_Decimals.Text = '' then
      TblDef.Fields[ListBox1.ItemIndex+1].Decimals := -1
    else
      TblDef.Fields[ListBox1.ItemIndex+1].Decimals := StrToInt(e_Decimals.Text);
  end;
end;

procedure Tfrm_QuickBuilder.e_FieldNameChange(Sender: TObject);
begin
     if ListBox1.ItemIndex > -1 then
     begin
       Listbox1.Items[Listbox1.ItemIndex] := e_FieldName.Text;
       TblDef.Fields[ListBox1.ItemIndex+1].FieldName := e_FieldName.Text;
     end;
end;

procedure Tfrm_Quickbuilder.SetProps;
begin
 PropBuild := True;
 try
  e_FieldName.Text := TblDef.Fields[ListBox1.ItemIndex+1].FieldName;
  e_Caption.Text := TblDef.Fields[Listbox1.ItemIndex+1].Caption;
  cb_DataType.ItemIndex := Ord(TblDef.Fields[ListBox1.ItemIndex+1].DataType);
  e_Decimals.Text := IntToStr(TblDef.Fields[ListBox1.ItemIndex+1].Decimals);
  cbx_Indexed.Checked := TblDef.Fields[ListBox1.ItemIndex+1].Indexed;
  cbx_Unique.Checked := TblDef.Fields[ListBox1.ItemIndex+1].Unique;
  cbx_Mandatory.Checked := TblDef.Fields[ListBox1.ItemIndex+1].Mandatory;
  cbx_Identifier.Checked := TblDef.Fields[ListBox1.ItemIndex+1].Identifier;
  cbx_Mulval.Checked := TblDef.Fields[ListBox1.ItemIndex+1].MultiVal;
 finally
   PropBuild := false;
 end;
 SetEnables;
end;

procedure Tfrm_QuickBuilder.SpeedButton3Click(Sender: TObject);
var
  FilePath: array [0 .. MAX_PATH] of Char;
  Dir : TArray<String>;
begin
   SHGetFolderPath(0, CSIDL_MyDocuments, 0, 0, FilePath);
   if SelectDirectory(FilePath, Dir, []) then
     e_Location.Text := Dir[0];
end;

procedure Tfrm_Quickbuilder.SetEnables;
var
  Indx : integer;
  IsPrntField : Boolean;
begin
  Indx := -1;
  if Table <> nil then
    Indx := Table.Fields.IndexOf(TblDef.Fields[ListBox1.ItemIndex+1].FieldName);
  if Indx = -1 then
  begin
    e_FieldName.Enabled := True;
    cbx_MandaTory.Enabled := True;
    cbx_Indexed.Enabled := True;
    cbx_Unique.Enabled := True;
    cbx_Identifier.Enabled := True;
    cbx_MulVal.Enabled := True;
  end
  else
  begin
    IsPrntField := (Table.Fields.Items[Indx].LookupType = looParentField);
    e_FieldName.Enabled := not IsPrntField and (Table.Fields.Items[Indx].FieldType <> ftSystemData);

    cbx_MandaTory.Enabled := e_FieldName.Enabled
    and not (Table.Fields.Items[Indx].FieldType in[ftCalculated, ftStoredEvaluation]);

    cbx_Indexed.Enabled := not IsPrntField and
      not (Table.Fields.Items[Indx].FieldType in[ftCalculated, ftStoredEvaluation])
      and not SameText(Table.Fields.Items[Indx].FieldName, 'SYS_LinkID');

    cbx_Unique.Enabled := not IsPrntField and
     not (Table.Fields.Items[Indx].FieldType in[ftCalculated, ftStoredEvaluation])
     and not SameText(Table.Fields.Items[Indx].FieldName, 'SYS_LinkID')
     and not SameText(Table.Fields.Items[Indx].FieldName, 'SYS_Level')
     and not SameText(Table.Fields.Items[Indx].FieldName, 'SYS_SiblingIndex') ;

     cbx_Identifier.Enabled := cbx_Unique.Enabled and
      not (Table.Fields.Items[Indx].DataType in [dtText, dtLongText]);

     e_Decimals.Enabled := not IsPrntField and (Table.Fields.Items[Indx].DataType = dtNumber)
     and (Table.Fields.Items[Indx].LookupType = looNone)
     and (Table.Fields.Items[Indx].FieldType = ftUserData);

     cb_DataType.Enabled := not IsPrntField and (Table.Fields.Items[Indx].FieldType = ftUserData)
     and (Table.Fields.Items[Indx].LookupType = looNone)
     and Table.IsTableEmpty;

     cbx_MulVal.Enabled := not IsPrntField and (Table.Fields.Items[Indx].DataType in [dtString, dtResource])
     and (Table.Fields.Items[Indx].FieldType = ftUserData);
  end;
end;

function Tfrm_Quickbuilder.FieldsChanged : TTableDef;
var
  i : integer;
  T : TILTable;
  Ind : integer;
begin
    Result.FieldCount := 0;
    if cb_Table.DroppedDown and not (cb_Table.Style = csSimple) then
      Exit;
    T := TMan.TableOf(cb_Table.Text);
    for i := 1 to TblDef.FieldCount do
    begin
       if TblDef.Fields[i].IsDeleted or (TblDef.Fields[i].FieldID = -1)then
       begin
          Inc(Result.FieldCount);
          Result.Fields[Result.FieldCount] := TblDef.Fields[i];
          Continue;
       end;

       Ind := T.Fields.IndexOfID(TblDef.Fields[i].FieldID);
       if not SameText(TblDef.Fields[i].FieldName, T.Fields.Items[Ind].FieldName) then
       begin
          Inc(Result.FieldCount);
          Result.Fields[Result.FieldCount] := TblDef.Fields[i];
       end
       else if not AnsiSameText(TblDef.Fields[i].Caption, T.Fields.Items[Ind].ReportCaption) then
       begin
          Inc(Result.FieldCount);
          Result.Fields[Result.FieldCount] := TblDef.Fields[i];
       end
       else if (TblDef.Fields[i].DataType <> T.Fields.Items[Ind].DataType) then
       begin
          Inc(Result.FieldCount);
          Result.Fields[Result.FieldCount] := TblDef.Fields[i];
       end
       else if (TblDef.Fields[i].Decimals <> T.Fields.Items[Ind].Decimals) then
       begin
          Inc(Result.FieldCount);
          Result.Fields[Result.FieldCount] := TblDef.Fields[i];
       end
       else if (TblDef.Fields[i].Indexed <> T.Fields.Items[Ind].Indexed) then
       begin
          Inc(Result.FieldCount);
          Result.Fields[Result.FieldCount] := TblDef.Fields[i];
       end
       else if (TblDef.Fields[i].Unique <> T.Fields.Items[Ind].Unique) then
       begin
          Inc(Result.FieldCount);
          Result.Fields[Result.FieldCount] := TblDef.Fields[i];
       end
       else if (TblDef.Fields[i].Mandatory <> T.Fields.Items[Ind].Mandatory) then
       begin
          Inc(Result.FieldCount);
          Result.Fields[Result.FieldCount] := TblDef.Fields[i];
       end
       else if (TblDef.Fields[i].Identifier <> T.Fields.Items[Ind].Identifier) then
       begin
          Inc(Result.FieldCount);
          Result.Fields[Result.FieldCount] := TblDef.Fields[i];
       end;;

    end;
end;


end.
